/*-----------------------------------------------------------------------------
 * $RCSfile: Block.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: Block.c,v 1.41 2002/03/27 01:15:42 bandou Exp $";
#endif  /* _DEBUG */

#include <OpenSOAP/Serializer.h>
#include "Block.h"
#include "XMLAttr.h"
#include "XMLNamespace.h"

/*
=begin
= OpenSOAP SOAP Block class
=end
 */

/*
=begin
--- function#OpenSOAPBlockCreate(soap_block)
    OpenSOAP XML Element instance create

    :Parameters
      :OpenSOAPBlockPtr * [out] ((|soap_block|))
        XML Element pointer
    :Return Value
      :int
	    Return Code Is ErrorCode
    :Note
=end
 */
extern
int
OpenSOAPBlockCreate(OpenSOAPBlockPtr * /* [out] */ soap_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( soap_block ){
		ret = OpenSOAPXMLElmCreate( (OpenSOAPXMLElmPtr*)soap_block );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
	}
	else{
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetValueString(soap_block, type_name, value)
    Get Value of SOAP Block(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP SOAP Block.
      :OpenSOAPStringPtr [in] ((|type_name|))
        Type Name
      :void * [out] ((|value|))
        Strage Buffer Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockGetValueString(OpenSOAPBlockPtr /* [in] */ soap_block,
                        OpenSOAPStringPtr /* [in] */ type_name,
                        void * /* [out] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetValueString((OpenSOAPXMLElmPtr)soap_block,
									type_name,
									value);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetValueMB(soap_block, type_name, value)
    Get Value of SOAP Block(MB)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP SOAP Block.
      :const char * [in] ((|type_name|))
        Type Name
      :void * [out] ((|value|))
        Strage Buffer Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockGetValueMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                        const char * /* [in] */ type_name,
                        void * /* [out] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetValueMB((OpenSOAPXMLElmPtr)soap_block,
									type_name,
									value);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetValueWC(soap_block, type_name, value)
    Get Value of SOAP Block(WC)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP SOAP Block.
      :const wchar_t * [in] ((|type_name|))
        Type Name
      :void * [out] ((|value|))
        Strage Buffer Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockGetValueWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                        const wchar_t * /* [in] */ type_name,
                        void * /* [out] */ value) {
	OpenSOAPStringPtr type_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(type_name, &type_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockGetValueString(soap_block, type_name_str, value);
		OpenSOAPStringRelease(type_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetValueMB(soap_block, type_name, value)
    Set Value of SOAP Block(MB)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP SOAP Block.
      :const char * [in] ((|type_name|))
        Type Name
      :void * [in] ((|value|))
        Strage Buffer Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
      Valuetype_nameSerializeʂsoap_block֊i[B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockSetValueMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                        const char * /* [in] */ type_name,
                        void * /* [in] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmSetValueMB( (OpenSOAPXMLElmPtr)soap_block, type_name, value );

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetValueWC(soap_block, type_name, value)
    Set Value of SOAP Block(WC)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP SOAP Block.
      :const wchar_t * [in] ((|type_name|))
        Type Name
      :void * [in] ((|value|))
        Strage Buffer Pointer
    :Return value
      :int
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
      Valuetype_nameSerializeʂsoap_block֊i[B
      
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockSetValueWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                         const wchar_t * /* [in] */ type_name,
                         void * /* [in] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmSetValueWC( (OpenSOAPXMLElmPtr)soap_block, type_name, value );

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetNamespaceMB(soap_block, ns_uri, ns_prefix)
    Set Namespace(MB)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|ns_uri|))
        Namespace URI
      :const char * [in] ((|ns_prefix|))
        Namespace Prefix
    :Return Value
      :int
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
      ns_prifix/ns_urisoap_block֊i[B
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetNamespaceMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ ns_uri,
                            const char * /* [in] */ ns_prefix) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmSetNamespaceMB( (OpenSOAPXMLElmPtr)soap_block, ns_uri, ns_prefix );

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetNamespaceWC(soap_block, ns_uri, ns_prefix)
    Set Namespace(WC)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|ns_uri|))
        Namespace URI
      :const wchar_t * [in] ((|ns_prefix|))
        Namespace Prefix
    :Return Value
      :int
	    Return Code Is ErrorCode
    :Note
      (2001/08/28)
      ns_prifix/ns_urisoap_block֊i[B
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetNamespaceWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ ns_uri,
                            const wchar_t * /* [in] */ ns_prefix) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmSetNamespaceWC( (OpenSOAPXMLElmPtr)soap_block, ns_uri, ns_prefix );

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetNamespace(soap_block, ns)
    Get Namespace

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPXMLNamespacePtr * [out] ((|ns|))
        Namespace 
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetNamespace(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                          OpenSOAPXMLNamespacePtr * /* [out] */ ns) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmGetNamespace( (OpenSOAPXMLElmPtr)soap_block, ns );

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockIsSameNamespaceString(soap_block, ns_uri, is_same_uri)
    Judge Namespace(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|ns_uri|))
        Namespace URI.
      :int * [out] ((|is_same_uri|))
        judge result buffer.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockIsSameNamespaceString(OpenSOAPBlockPtr /* [in] */ soap_block,
                           OpenSOAPStringPtr /* [in] */ ns_uri,
                           int * /* [out] */ is_same_uri) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	int compare_result;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( soap_block->blockElm.super.thisNamespace != NULL ){
		ret = OpenSOAPStringCompare(soap_block->blockElm.super.thisNamespace->uri,
									  ns_uri,
									  &compare_result);
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
		if( compare_result == 0 ){
			*is_same_uri = 1;
			ret = OPENSOAP_NO_ERROR;
		}
		else{
			*is_same_uri = 0;
			ret = OPENSOAP_NO_ERROR;
		}
	}
	else{
		*is_same_uri = 0;
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockIsSameNamespaceMB(soap_block, ns_uri, is_same_uri)
    Judge Namespace(MB)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|ns_uri|))
        Namespace URI.
      :int * [out] ((|is_same_uri|))
        judge result buffer.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockIsSameNamespaceMB(OpenSOAPBlockPtr /* [in] */ soap_block,
                           const char * /* [in] */ ns_uri,
                           int * /* [out] */ is_same_uri) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	int compare_result;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( soap_block->blockElm.super.thisNamespace != NULL ){
		ret = OpenSOAPStringCompareMB(soap_block->blockElm.super.thisNamespace->uri,
									  ns_uri,
									  &compare_result);
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
		if( compare_result == 0 ){
			*is_same_uri = 1;
			ret = OPENSOAP_NO_ERROR;
		}
		else{
			*is_same_uri = 0;
			ret = OPENSOAP_NO_ERROR;
		}
	}
	else{
		*is_same_uri = 0;
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockIsSameNamespaceWC(soap_block, ns_uri, is_same_uri)
    Judge Namespace(WC)

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|ns_uri|))
        Namespace URI.
      :int * [out] ((|is_same_uri|))
        judge result buffer.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockIsSameNamespaceWC(OpenSOAPBlockPtr /* [in] */ soap_block,
                               const wchar_t * /* [in] */ ns_uri,
                               int * /* [out] */ is_same_uri) {
	OpenSOAPStringPtr ns_uri_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(ns_uri, &ns_uri_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockIsSameNamespaceString(soap_block, ns_uri_str, is_same_uri);
		OpenSOAPStringRelease(ns_uri_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockAddAttributeString(soap_block, attr_name, attr_type, attr_value, attr)
    Add and Set Attribute(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|attr_name|))
        Attribute Name
      :OpenSOAPStringPtr [in] ((|attr_type|))
        Attribute Type
      :void * [in] ((|attr_value|))
        Attribute Value
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockAddAttributeString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            OpenSOAPStringPtr /* [in] */ attr_name,
                            OpenSOAPStringPtr /* [in] */ attr_type,
                            void * /* [in] */ attr_value,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeString( (OpenSOAPXMLElmPtr)soap_block,
									   attr_name,
										attr_type,
										attr_value,
										attr);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockAddAttributeMB(soap_block, attr_name, attr_type, attr_value, attr)
    Add and Set Attribute(MB)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|attr_name|))
        Attribute Name
      :const char * [in] ((|attr_type|))
        Attribute Type
      :void * [in] ((|attr_value|))
        Attribute Value
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockAddAttributeMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ attr_name,
                            const char * /* [in] */ attr_type,
                            void * /* [in] */ attr_value,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										attr_name,
										attr_type,
										attr_value,
										attr);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockAddAttributeWC(soap_block, attr_name, attr_type, attr_value, attr)
    Add and Set Attribute(WC)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|attr_name|))
        Attribute Name
      :const wchar_t * [in] ((|attr_type|))
        Attribute Type
      :void * [in] ((|attr_value|))
        Attribute Value
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockAddAttributeWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ attr_name,
                            const wchar_t * /* [in] */ attr_type,
                            void * /* [in] */ attr_value,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
	OpenSOAPStringPtr attr_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(attr_name, &attr_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr attr_type_str = NULL;
		int ret = OpenSOAPStringCreateWithWC(attr_type, &attr_type_str);
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPBlockAddAttributeString(soap_block, attr_name_str, attr_type_str, attr_value, attr);
			OpenSOAPStringRelease(attr_type_str);
		}
		OpenSOAPStringRelease(attr_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetAttributeString(soap_block, attr_name, attr)
    Get Value of Attribute(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|attr_name|))
        Attribute Name
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockGetAttributeString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            OpenSOAPStringPtr /* [in] */ attr_name,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !attr ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}
	else{
		ret = OpenSOAPXMLElmGetAttributeString( (OpenSOAPXMLElmPtr)soap_block,
											attr_name,
											attr );
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetAttributeMB(soap_block, attr_name, attr)
    Get Value of Attribute(MB)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|attr_name|))
        Attribute Name
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetAttributeMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ attr_name,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !attr ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}
	else{
		ret = OpenSOAPXMLElmGetAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
											attr_name,
											attr );
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetAttributeWC(soap_block, attr_name, attr)
    Get Value of Attribute

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|attr_name|))
        Attribute Name
      :OpenSOAPXMLAttrPtr * [out] ((|attr|))
        OpenSOAP XML Attribute
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetAttributeWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ attr_name,
                            OpenSOAPXMLAttrPtr * /* [out] */ attr) {
	OpenSOAPStringPtr attr_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(attr_name, &attr_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockGetAttributeString(soap_block, attr_name_str, attr);
		OpenSOAPStringRelease(attr_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildString(soap_block, elm_name, xml_elm)
    Oɍvq XML Element ̎擾B
    v̂ꍇ́Aŏ̗vfoB
    (OpenSOAPString)
    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|elm_name|))
        Name of XML Element
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockGetChildString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        OpenSOAPStringPtr /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !xml_elm ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildString( (OpenSOAPXMLElmPtr)soap_block,
									elm_name,
									xml_elm);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildMB(soap_block, elm_name, xml_elm)
    Oɍvq XML Element ̎擾B
    v̂ꍇ́Aŏ̗vfoB

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|elm_name|))
        Name of XML Element
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetChildMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        const char * /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !xml_elm ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetChildMB( (OpenSOAPXMLElmPtr)soap_block,
									elm_name,
									xml_elm);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildWC(soap_block, elm_name, xml_elm)
    Oɍvq XML Element ̎擾B
    v̂ꍇ́Aŏ̗vfoB

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|elm_name|))
        Name of Child XML Element
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetChildWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        const wchar_t * /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
	OpenSOAPStringPtr elm_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(elm_name, &elm_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockGetChildString(soap_block, elm_name_str, xml_elm);
		OpenSOAPStringRelease(elm_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildValueString(soap_block, p_name, type_name, value)
    l̎擾(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|p_name|))
        p[^
      :OpenSOAPStringPtr [in] ((|type_name|))
        ^
      :void * [out] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockGetChildValueString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             OpenSOAPStringPtr /* [in] */ p_name,
                             OpenSOAPStringPtr /* [in] */ type_name,
                             void * /* [out] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLElmPtr  xml_elm = NULL;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPBlockGetChildString( soap_block, p_name, &xml_elm );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	if( xml_elm != NULL ){
		ret = OpenSOAPXMLElmGetValueString( xml_elm, type_name, value );
	}
	else{
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildValueMB(soap_block, p_name, type_name, value)
    l̎擾

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|p_name|))
        p[^
      :const char * [in] ((|type_name|))
        ^
      :void * [out] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetChildValueMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             const char * /* [in] */ p_name,
                             const char * /* [in] */ type_name,
                             void * /* [out] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLElmPtr  xml_elm = NULL;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPBlockGetChildMB( soap_block, p_name, &xml_elm );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

	if( xml_elm != NULL ){
		ret = OpenSOAPXMLElmGetValueMB( xml_elm, type_name, value );
	}
	else{
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildValueWC(soap_block, p_name, type_name, value)
    l̎擾

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|p_name|))
        p[^
      :const wchar_t * [in] ((|type_name|))
        ^
      :void * [out] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetChildValueWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             const wchar_t * /* [in] */ p_name,
                             const wchar_t * /* [in] */ type_name,
                             void * /* [out] */ value) {
	OpenSOAPStringPtr p_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(p_name, &p_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr type_name_str = NULL;
		int ret = OpenSOAPStringCreateWithWC(type_name, &type_name_str);
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPBlockGetChildValueString(soap_block, p_name_str, type_name_str, value);
			OpenSOAPStringRelease(type_name_str);
		}
		OpenSOAPStringRelease(p_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetChildValueString(soap_block, p_name, type_name, value)
    l̐ݒ(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|p_name|))
        p[^
      :OpenSOAPStringPtr [in] ((|type_name|))
        ^
      :void * [in] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockSetChildValueString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             OpenSOAPStringPtr /* [in] */ p_name,
                             OpenSOAPStringPtr /* [in] */ type_name,
                             void * /* [in] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	int compare_result;

	OpenSOAPXMLNodePtr cld_node = NULL;
	OpenSOAPXMLNodePtr cld_pre = NULL;

	OpenSOAPSerializerFunc serializer = NULL;


	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	cld_node = soap_block->blockElm.super.children;

	/* If there are any children in the block, check the named child*/
	if( cld_node != NULL ){
		while( cld_node != NULL ){
			ret = OpenSOAPStringCompare(cld_node->name,p_name,&compare_result);
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}

			if( compare_result == 0 ){
				if( cld_node->value == NULL ){
					ret = OpenSOAPStringCreate( &(cld_node->value) );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
				}
				
				ret = OpenSOAPGetSerializer(type_name, &serializer);
				if (OPENSOAP_SUCCEEDED(ret)) {
					ret = serializer(value, (cld_node->value));
					return ret;
				}
				else{
					return ret;
				}
			}
			cld_pre = cld_node;
			cld_node = cld_node->next;
		}

		if( cld_node == NULL ){
			ret = OpenSOAPXMLElmAddChildString( (OpenSOAPXMLElmPtr)soap_block,
											p_name,
											(OpenSOAPXMLElmPtr *)(&cld_node) );
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}

			ret = OpenSOAPXMLElmSetValueString( (OpenSOAPXMLElmPtr)cld_node, type_name, value );

			cld_pre->next = cld_node;
		}
	}
	else{	
		/* If there is not the named child, make a new child named by p_name*/
		ret = OpenSOAPXMLElmAddChildString( (OpenSOAPXMLElmPtr)soap_block,
										p_name,
										(OpenSOAPXMLElmPtr *)(&cld_node) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
		ret = OpenSOAPXMLElmSetValueString( (OpenSOAPXMLElmPtr)(cld_node), type_name, value );

		soap_block->blockElm.super.children = cld_node;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetChildValueMB(soap_block, p_name, type_name, value)
    l̐ݒ

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|p_name|))
        p[^
      :const char * [in] ((|type_name|))
        ^
      :void * [in] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetChildValueMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             const char * /* [in] */ p_name,
                             const char * /* [in] */ type_name,
                             void * /* [in] */ value) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	int compare_result;

	OpenSOAPXMLNodePtr cld_node = NULL;
	OpenSOAPXMLNodePtr cld_pre = NULL;

	OpenSOAPSerializerFunc serializer = NULL;


	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	cld_node = soap_block->blockElm.super.children;

	/* If there are any children in the block, check the named child*/
	if( cld_node != NULL ){
		while( cld_node != NULL ){
			ret = OpenSOAPStringCompareMB(cld_node->name,p_name,&compare_result);
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}

			if( compare_result == 0 ){
				if( cld_node->value == NULL ){
					ret = OpenSOAPStringCreate( &(cld_node->value) );
					if( OPENSOAP_FAILED(ret) ){
						return ret;
					}
				}
				
				ret = OpenSOAPGetSerializerMB(type_name, &serializer);
				if (OPENSOAP_SUCCEEDED(ret)) {
					ret = serializer(value, (cld_node->value));
					return ret;
				}
				else{
					return ret;
				}
			}
			cld_pre = cld_node;
			cld_node = cld_node->next;
		}

		if( cld_node == NULL ){
			ret = OpenSOAPXMLElmAddChildMB( (OpenSOAPXMLElmPtr)soap_block,
											p_name,
											(OpenSOAPXMLElmPtr *)(&cld_node) );
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}

			ret = OpenSOAPXMLElmSetValueMB( (OpenSOAPXMLElmPtr)cld_node, type_name, value );

			cld_pre->next = cld_node;
		}
	}
	else{	
		/* If there is not the named child, make a new child named by p_name*/
		ret = OpenSOAPXMLElmAddChildMB( (OpenSOAPXMLElmPtr)soap_block,
										p_name,
										(OpenSOAPXMLElmPtr *)(&cld_node) );
		if( OPENSOAP_FAILED(ret) ){
			return ret;
		}
		ret = OpenSOAPXMLElmSetValueMB( (OpenSOAPXMLElmPtr)(cld_node), type_name, value );

		soap_block->blockElm.super.children = cld_node;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetChildValueWC(soap_block, p_name, type_name, value)
    l̎擾

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|p_name|))
        p[^
      :const wchar_t * [in] ((|type_name|))
        ^
      :void * [in] ((|value|))
        li[obt@ւ̃|C^
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetChildValueWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                             const wchar_t * /* [in] */ p_name,
                             const wchar_t * /* [in] */ type_name,
                             void * /* [in] */ value) {
	OpenSOAPStringPtr p_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(p_name, &p_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		OpenSOAPStringPtr type_name_str = NULL;
		int ret = OpenSOAPStringCreateWithWC(type_name, &type_name_str);
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = OpenSOAPBlockSetChildValueString(soap_block, p_name_str, type_name_str, value);
			OpenSOAPStringRelease(type_name_str);
		}
		OpenSOAPStringRelease(p_name_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetNextChild(soap_block, xml_elm)
    

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPXMLElmPtr * [in, out] ((|xml_elm|))
        XML Element i[ϐB
        *xml_elm  NULL ̏ꍇAŏ̎qB
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetNextChild(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                          OpenSOAPXMLElmPtr * /* [in, out] */ xml_elm) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !xml_elm ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetNextChild( (OpenSOAPXMLElmPtr)soap_block,
									  xml_elm );
    return ret;
}

/*
=begin
--- function#OpenSOAPBlockAddChildString(soap_block, elm_name, xml_elm)
    Add of Child XML Element(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|elm_name|))
        q XML Element ̖OB
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockAddChildString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        OpenSOAPStringPtr /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmAddChildString( (OpenSOAPXMLElmPtr)soap_block,
									elm_name,
									xml_elm );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

    return ret;
}


/*
=begin
--- function#OpenSOAPBlockAddChildMB(soap_block, elm_name, xml_elm)
    Add of Child XML Element(MB)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|elm_name|))
        q XML Element ̖OB
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockAddChildMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        const char * /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	ret = OpenSOAPXMLElmAddChildMB( (OpenSOAPXMLElmPtr)soap_block,
									elm_name,
									xml_elm );
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockAddChildWC(soap_block, elm_name, xml_elm)
    Add of Child XML Element(WC)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|elm_name|))
        Name of XML Element
      :OpenSOAPXMLElmPtr * [out] ((|xml_elm|))
        OpenSOAP XML Element.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockAddChildWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                        const wchar_t * /* [in] */ elm_name,
                        OpenSOAPXMLElmPtr * /* [out] */ xml_elm) {
	OpenSOAPStringPtr elm_name_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(elm_name, &elm_name_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockAddChildString(soap_block, elm_name_str, xml_elm);
		OpenSOAPStringRelease(elm_name_str);
	}
	return ret;
}


/*
=begin
--- function#OpenSOAPBlockGetMustunderstandAttr(soap_block, must_std)
    Get mustunderstand attribute.

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        SOAP Block.
      :int * [out] ((|must_std|))
        mustunderstand return buffer pointer
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetMustunderstandAttr(OpenSOAPBlockPtr /* [in] */ soap_block,
                                   int * /* [out] */ must_std) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	OpenSOAPXMLAttrPtr attr;
	OpenSOAPDeserializerFunc deserializer = NULL;
	
	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										"mustUnderstand",
										&attr);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		ret = OpenSOAPGetDeserializerMB("int", &deserializer);
		if (OPENSOAP_SUCCEEDED(ret)) {
			ret = deserializer(((OpenSOAPXMLNodePtr)attr)->value, must_std);
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}
		}
	}
    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetMustunderstandAttr(soap_block)
    Set mustunderstand attribute.

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetMustunderstandAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLAttrPtr attr_next;
	int attr_value = 1;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										"mustUnderstand",
										"int",
										&attr_value,
										&attr_next);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockClearMustunderstandAttr(soap_block)
    Clear mustunderstand attribute.

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockClearMustunderstandAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetActorAttr(soap_block, actor_url)
    Set mustunderstand attribute.

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr * [out] ((|actor_url|))
        actor attribute value.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetActorAttr(OpenSOAPBlockPtr /* [in] */ soap_block,
                          OpenSOAPStringPtr * /* [out] */ actor_url) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	OpenSOAPXMLAttrPtr attr;
	OpenSOAPDeserializerFunc deserializer = NULL;
	
	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPXMLElmGetAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										"actor",
										&attr);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		*actor_url = attr->super.value;
		ret = OPENSOAP_NO_ERROR;
	}
    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetActorAttrString(soap_block, actor_url)
    Set actor attribute.(OpenSOAPString)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :OpenSOAPStringPtr [in] ((|actor_url|))
        actor attribute value.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
      
=end
 */
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPBlockSetActorAttrString(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            OpenSOAPStringPtr /* [in] */ actor_url) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLAttrPtr attr_next = NULL;

	OpenSOAPStringPtr opensoap_actor_uri;
	OpenSOAPStringPtr opensoap_actor_str = NULL;
	OpenSOAPStringPtr opensoap_actor_type_str = NULL;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPStringDuplicate( actor_url, &opensoap_actor_uri );
	if( OPENSOAP_FAILED(ret) ) {
		return ret;
	}

	ret = OpenSOAPStringCreateWithMB( "actor", &opensoap_actor_str );
	if( OPENSOAP_FAILED(ret) ) {
		OpenSOAPStringRelease(opensoap_actor_uri);
		return ret;
	}
	ret = OpenSOAPStringCreateWithMB( "string", &opensoap_actor_type_str );
	if( OPENSOAP_FAILED(ret) ) {
		OpenSOAPStringRelease(opensoap_actor_str);
		OpenSOAPStringRelease(opensoap_actor_uri);
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeString( (OpenSOAPXMLElmPtr)soap_block,
										opensoap_actor_str,
										opensoap_actor_type_str,
										&opensoap_actor_uri,
										&attr_next);
	OpenSOAPStringRelease(opensoap_actor_type_str);
	OpenSOAPStringRelease(opensoap_actor_str);
	OpenSOAPStringRelease(opensoap_actor_uri);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetActorAttrMB(soap_block, actor_url)
    Set actor attribute.(MB)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const char * [in] ((|actor_url|))
        actor attribute value.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetActorAttrMB(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const char * /* [in] */ actor_url) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLAttrPtr attr_next = NULL;

	OpenSOAPStringPtr opensoap_actor_uri;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPStringCreateWithMB( actor_url, &opensoap_actor_uri );
	if( OPENSOAP_FAILED(ret) ) {
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										"actor",
										"string",
										&opensoap_actor_uri,
										&attr_next);

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetActorAttrWC(soap_block, actor_url)
    Set actor attribute.(WC)

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :const wchar_t * [in] ((|actor_url|))
        actor attribute value.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetActorAttrWC(OpenSOAPBlockPtr /* [in, out] */ soap_block,
                            const wchar_t * /* [in] */ actor_url) {
	OpenSOAPStringPtr actor_url_str = NULL;
	int ret = OpenSOAPStringCreateWithWC(actor_url, &actor_url_str);
	if (OPENSOAP_SUCCEEDED(ret)) {
		ret = OpenSOAPBlockSetActorAttrString(soap_block, actor_url_str);
		OpenSOAPStringRelease(actor_url_str);
	}
	return ret;
}

/*
=begin
--- function#OpenSOAPBlockClearActorAttr(soap_block)
    Clear actor attribute.

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockClearActorAttr(OpenSOAPBlockPtr /* [in, out] */ soap_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockSetActorAttrNext(soap_block)
    Set actor attribute to next.

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockSetActorAttrNext(OpenSOAPBlockPtr /* [in, out] */ soap_block) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	OpenSOAPXMLAttrPtr attr_next;
	char actor_uri[100] = "http://www.w3.org/2001/06/soap-envelope/actor/next";

	OpenSOAPStringPtr opensoap_actor_uri;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	ret = OpenSOAPStringCreateWithMB( actor_uri, &opensoap_actor_uri );
	if( OPENSOAP_FAILED(ret) ) {
		return ret;
	}

	ret = OpenSOAPXMLElmAddAttributeMB( (OpenSOAPXMLElmPtr)soap_block,
										"actor",
										"string",
										&opensoap_actor_uri,
										&attr_next);
	if( OPENSOAP_FAILED(ret) ){
		return ret;
	}
	else{
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockIsActorAttrNext(soap_block, is_actor_next)
    Is actor attribute to next.

    :Parameters
      :OpenSOAPBlockPtr [in, out] ((|soap_block|))
        SOAP Block.
      :int * [out] ((|is_actor_next|))
        judge result.
    :Return Value
      :int
	    Return Code Is ErrorCode
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockIsActorAttrNext(OpenSOAPBlockPtr /* [in] */ soap_block,
                             int * /* [out] */ is_actor_next) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
	int compare_result;
	char actor_uri[70] = "http://www.w3.org/2001/06/soap-envelope/actor/next";

	OpenSOAPXMLAttrPtr attr_next = NULL;

	if( !soap_block ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
		return ret;
	}

	if( soap_block->blockElm.xmlAttrList != NULL ){
		attr_next = soap_block->blockElm.xmlAttrList;
		while( attr_next != NULL ){
			ret = OpenSOAPStringCompareMB(attr_next->super.name,
										  "actor",
										  &compare_result);
			if( OPENSOAP_FAILED(ret) ){
				return ret;
			}
			if( compare_result == 0 ){
				ret = OpenSOAPStringCompareMB(attr_next->super.value,
											  actor_uri,
											  &compare_result);
				if( OPENSOAP_FAILED(ret) ){
					return ret;
				}

				if( compare_result == 0 ){
					*is_actor_next = 1;
					ret = OPENSOAP_NO_ERROR;
					return ret;
				}
				else{
					*is_actor_next = 0;
					ret = OPENSOAP_NO_ERROR;
					return ret;
				}
			}
			else{
				attr_next = (OpenSOAPXMLAttrPtr)(attr_next->super.next);
			}
		}
	}
	else{
		*is_actor_next = 0;
		ret = OPENSOAP_NO_ERROR;
	}

    return ret;
}

/*
=begin
--- function#OpenSOAPBlockGetName(block, name)
    Getting of SOAP Block Name

    :Parameters
      :OpenSOAPBlockPtr [in] ((|block|))
        SOAP Block Pointer.
      :OpenSOAPStringPtr * [out] ((|name|))
        Result Name of SOAP Block.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPBlockGetName(OpenSOAPBlockPtr /* [in] */ block,
                     OpenSOAPStringPtr * /* [out] */ name) {
    int ret = OpenSOAPXMLNodeGetName((OpenSOAPXMLNodePtr)block,
                                     name);

    return ret;
}



/*
=begin
--- function#OpenSOAPBlockGetCharEncodingString(soap_block, ch_enc, b_ary)
    Soap block character encoding output

    :Parameters
      :OpenSOAPBlockPtr [in] ((|soap_block|))
        OpenSOAP Block.
      :const char  * [in] ((|ch_enc|))
        character encoding. i.e. "EUC-JP", "SHIFT-JIS", "UTF-8"
      :OpenSOAPByteArrayPtr [out] ((|b_ary|))
        Result Buffer        
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPBlockGetCharEncodingString(OpenSOAPBlockPtr /* [in] */ soap_block,
                                    const char * /* [in] */ ch_enc,
                                    OpenSOAPByteArrayPtr /* [out] */ b_ary) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;

	if( !soap_block || !b_ary ){
		ret = OPENSOAP_PARAMETER_BADVALUE;
	}
	else{
		ret = OpenSOAPXMLElmGetCharEncodingString( (OpenSOAPXMLElmPtr)soap_block,
													ch_enc,
													b_ary );
	}

    return ret;
}
