/*-----------------------------------------------------------------------------
 * $RCSfile: Stream.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: Stream.c,v 1.5 2001/12/18 02:00:50 bandou Exp $";
#endif  /* _DEBUG */

#include "Stream.h"

/*
=begin
= OpenSOAP Stream class
=end
 */

extern
int
OPENSOAP_API
OpenSOAPStreamReleaseMembers(OpenSOAPStreamPtr s) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (s) {
        s->readFunc = NULL;
        s->writeFunc = NULL;
        ret = OpenSOAPObjectReleaseMembers((OpenSOAPObjectPtr)s);
    }

    return ret;
}

extern
int
OPENSOAP_API
OpenSOAPStreamInitialize(OpenSOAPStreamPtr /* [in] */ s,
                         OpenSOAPObjectFreeFunc /* [in] */ free_func,
                         OpenSOAPStreamReadFunc  /* [in] */ read_func,
                         OpenSOAPStreamWriteFunc /* [in] */ write_func) {
    OpenSOAPObjectPtr obj = (OpenSOAPObjectPtr)s;
    int ret = OpenSOAPObjectInitialize(obj, free_func);
    if (OPENSOAP_SUCCEEDED(ret)) {
        s->readFunc  = read_func;
        s->writeFunc = write_func;
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStreamRelease(s)
    OpenSOAP Stream JB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamRelease(OpenSOAPStreamPtr /* [in, out] */ s) {
    return OpenSOAPObjectRelease((OpenSOAPObjectPtr)s);
}

/*
=begin
--- function#OpenSOAPStreamRead(s, buf, buf_sz)
    OpenSOAP Stream ǂݍށB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
      :unsigned char * [out] buf,
        ǂݍ݃obt@̐擪|C^B
      :size_t * [in, out] buf_sz
        ǂݍ݃obt@̃TCYBďo͓ǂݍ񂾃TCYԂB
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamRead(OpenSOAPStreamPtr /* [in] */ s,
                   unsigned char * /* [out] */ buf,
                   size_t * /* [in, out] */ buf_sz) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (s && s->readFunc) {
        ret = (s->readFunc)(s, buf, buf_sz);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStreamWrite(s, buf, buf_sz)
    OpenSOAP Stream ֏oB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
      :const unsigned char * [in] ((|buf|))
        ݃f[^̐擪|C^B
      :size_t * [in, out] buf_sz
        ݃f[^̃TCYBďo͏񂾃TCYԂB
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamWrite(OpenSOAPStreamPtr /* [in] */ s,
                    const unsigned char * /* [in] */ buf,
                    size_t * /* [in, out] */ buf_sz) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (s && s->writeFunc) {
        ret = (s->writeFunc)(s, buf, buf_sz);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStreamReadToDelm(s, delm, delm_sz, buf)
    OpenSOAP Stream f~^܂œǂݍށB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
      :const unsigned char * [in] ((|delm|))
        f~^f[^B
      :size_t [in] ((|delm_sz|))
        f~^f[^񒷂B
      :OpenSOAPByteArrayPtr [out] ((|buf|))
        ǂݍ݃obt@B
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamReadToDelm(OpenSOAPStreamPtr /* [in] */ s,
                         const unsigned char * /* [in] */ delm,
                         size_t /* [in] */ delm_sz,
                         OpenSOAPByteArrayPtr /* [out] */ buf) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    
    if (delm && delm_sz) {
        ret = OpenSOAPByteArrayClear(buf);
        if (OPENSOAP_SUCCEEDED(ret)) {
            const unsigned char * const delm_end = delm + delm_sz;
            const unsigned char *delm_i = delm;
        
            while (delm_i != delm_end) {
                unsigned char read_dat = '\x0';
                size_t read_sz = 1;
                ret =
                    OpenSOAPStreamRead(s, &read_dat, &read_sz);
                if (OPENSOAP_FAILED(ret) || !read_sz) {
                    break;
                }
                ret =
                    OpenSOAPByteArrayAppend(buf,
                                            &read_dat,
                                            1);
                if (OPENSOAP_FAILED(ret)) {
                    break;
                }
                if (*delm_i == read_dat) {
                    ++delm_i;
                }
                else {
                    delm_i = delm;
                }
            }
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStreamReadWithSize(s, sz, buf)
    OpenSOAP Stream wTCYǂݍށBsz  0 ̏ꍇA
    ǂݍ߂邾ǂݍށB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
      :size_t [in] ((|sz|))
        ǂݍ݃TCYB
      :OpenSOAPByteArrayPtr [out] ((|buf|))
        ǂݍ݃obt@B
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamReadWithSize(OpenSOAPStreamPtr /* [in] */ s,
                           size_t /* [in] */ sz,
                           OpenSOAPByteArrayPtr /* [out] */ buf) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    
    if (s) {
#define READ_BUF_SZ 128
        unsigned char read_buf[READ_BUF_SZ];
        size_t read_sum_sz = 0;
        ret = OpenSOAPByteArrayClear(buf);
        while (OPENSOAP_SUCCEEDED(ret)) {
            size_t read_sz
                = (sz && sz < read_sum_sz + READ_BUF_SZ)
                ? (sz - read_sum_sz) : READ_BUF_SZ;
            if (!read_sz) {
                break;
            }
            ret = OpenSOAPStreamRead(s,
                                     read_buf,
                                     &read_sz);
            if (OPENSOAP_SUCCEEDED(ret)) {
                if (!read_sz) {
                    break;
                }
                ret = OpenSOAPByteArrayAppend(buf,
                                              read_buf,
                                              read_sz);
                read_sum_sz += read_sz;
            }
        }
#undef READ_BUF_SZ
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStreamWriteByteArray(s, buf)
    OpenSOAP Stream ֏oB

    :Parameters
      :OpenSOAPStreamPtr  [in] ((|s|))
        OpenSOAP Stream
      :OpenSOAPByteArrayPtr [in] ((buf|))
        ݃f[^B
      :size_t * [out] ((|write_sz|))
        ݃TCYB
    :Returns
      :int
      
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStreamWriteByteArray(OpenSOAPStreamPtr /* [in] */ s,
                             OpenSOAPByteArrayPtr /* [in] */ buf,
                             size_t * /* [out] */ write_sz) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    const unsigned char *buf_beg = NULL;
    size_t sz = 0;

    if (!write_sz) {
        write_sz = &sz;
    }
    ret = OpenSOAPByteArrayGetBeginSizeConst(buf, &buf_beg, write_sz);
    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = OpenSOAPStreamWrite(s, buf_beg, write_sz);
    }

    return ret;
}
