/*-----------------------------------------------------------------------------
 * $RCSfile: String.c,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if (!defined(_WIN32)) || defined(_DEBUG)
static  const   char    CVS_ID[]        =       "$Id: String.c,v 1.98 2002/01/31 06:58:36 bandou Exp $";
#endif  /* _DEBUG */

#include "String.h"

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>

#if defined(HAVE_WCHAR_H) || defined(NONGCC_WIN32)
#include <wchar.h>
#endif

#ifdef _MSC_VER
#define vsnprintf _vsnprintf
#define vswprintf _vsnwprintf

#define HAVE_VSWPRINTF 1

#define wmemmove(dest, src, n) \
(wchar_t *)memmove((dest), (src), (n) * sizeof(wchar_t))
#define wmemcpy(dest, src, n) \
(wchar_t *)memcpy((dest), (src), (n) * sizeof(wchar_t))

#define	strcasecmp(s1, s2) _stricmp((s1), (s2))

#define HAVE_WCSCMP 1
#define HAVE_WCSCPY 1
#define HAVE_WCSLEN 1
#define HAVE_WCSNCAT 1
#define HAVE_WCSSTR 1
#define HAVE_WMEMCPY 1
#define HAVE_WMEMMOVE 1

#endif /* _MSC_VER */

#if !defined(HAVE_WCSCMP)
extern int wcscmp(const wchar_t *s1, const wchar_t *s2);
#endif /* !HAVE_WCSCMP */
#if !defined(HAVE_WCSCPY)
extern wchar_t *wcscpy(wchar_t *dest, const wchar_t *src);
#endif /* !HAVE_WCSCPY */
#if !defined(HAVE_WCSLEN)
size_t wcslen(const wchar_t *s);
#endif /* !HAVE_WCSLEN */
#if !defined(HAVE_WCSNCAT)
extern wchar_t *wcsncat(wchar_t *dest, const wchar_t *src, size_t n);
#endif /* !HAVE_WCSNCAT */
#if !defined(HAVE_WCSSTR)
extern wchar_t *wcsstr(const wchar_t *heystack, const wchar_t *needle);
#endif /* !HAVE_WCSSTR */
#if !defined(HAVE_WMEMCPY)
extern wchar_t *wmemcpy(wchar_t *dest, const wchar_t *src, size_t n);
#endif /* !HAVE_WMEMCPY */
#if !defined(HAVE_WMEMMOVE)
extern wchar_t *wmemmove(wchar_t *dest, const wchar_t *src, size_t n);
#endif /* !HAVE_WMEMMOVE */

#if defined(WITH_MBFUNCS)
/*
  temporary implemantation
 */
static
int
sblen(const char *s,
      size_t n) {
    int ret = 0;

    if (s) {
        const char *i = s;
        const char *e = s + n;
        ret = -1;
        for (; i != e && *i && (*i & 0x80); ++i) {
        }
        if (i == e || !*i) {
            ret = i - s;
        }
    }

    return ret;
}

static
size_t
wcstosbs(char *dest,
         const wchar_t *src,
         size_t n) {
    size_t ret = (size_t)(-1);
    if (src) {
        const wchar_t *i = src;
        const wchar_t *e = src + n;
        for (; *i && (*i < 0x80); ++i) {
            if (dest) {
                if (i == e) {
                    break;
                }
                *dest++ = (char)*i;
            }
        }
        if (i == e || !*i) {
            if (dest) {
                *dest = '\0';
            }
            ret = i - src;
        }
    }

    return ret;
}

static
size_t
sbstowcs(wchar_t *dest,
         const char *src,
         size_t n) {
    size_t ret = (size_t)(-1);
    if (src) {
        const char *i = src;
        const char *e = src + n;
        for (; *i && !(*i & 0x80); ++i) {
            if (dest) {
                if (i == e) {
                    break;
                }
                *dest++ = (wchar_t)*i;
            }
        }
        if (i == e || !*i) {
            if (dest) {
                *dest = L'\0';
            }
            ret = i - src;
        }
    }

    return ret;
}

#define mblen(s, n) sblen((s), (n))
#define mbstowcs(dest, src, n) sbstowcs((dest), (src), (n))
#define wcstombs(dest, src, n) wcstosbs((dest), (src), (n))

#endif

/*
=begin
= String.c
OpenSOAPString Implement file.
=end
*/

/*
=begin
--- function#XwcsFree(wcstr)

    :Parameters
      :wchar_t ** [out, in] ((|wcstr|))
        Wide Character string's pointer.
    :Return Value
      nothing.
=end
 */
static
void
XwcsFree(wchar_t ** /* [out, in] */ wcstr) {
    if (wcstr) {
        free(*wcstr);
        *wcstr = NULL;
    }
}

/*
=begin
--- function#XwcsRealloc(wcstr, wcstr_len)

    :Parameters
      :wchar_t * [in] ((|wcstr|))
        Wide character string pointer.
      :size_t [in] ((|wcstr_sz|))
        Wide Character string buffer size (include L'\0').
    :Return Value
      :wchar_t *
        allocatedbuffer
=end
 */
static
wchar_t *
XwcsRealloc(wchar_t * /* [in] */ wcstr,
            size_t /* [in] */ wcstr_sz) {
    wchar_t *ret = NULL;

    if (wcstr_sz) {
        ret = realloc(wcstr, wcstr_sz * sizeof(wchar_t));
    }

    return ret;
}

/*
=begin
--- function#XwcsAlloc(wcstr_len)

    :Parameters
      :size_t [in] ((|wcstr_sz|))
        Wide Character string buffer size (include L'\0')
    :Return Value
      :wchar_t *
        allocatedbuffer
=end
 */
#if 1
#define XwcsAlloc(wcstr_sz) XwcsRealloc(0, (wcstr_sz))
#else
static
wchar_t *
XwcsAlloc(size_t /* [out, in] */ wcstr_sz) {
    wchar_t *ret = NULL;

    if (wcstr_sz) {
        ret = malloc(sizeof(wchar_t) * wcstr_sz);
    }

    return ret;
}
#endif

/*
=begin
--- function#Xmbstowcs(mbstr, sz)

    :Parameters
      :const char * [in] ((|mbstr|))
        Multi Byte string
      :size_t * [in, out] ((|mbs_sz|))
        [in] Multi Byte string byte size
        [out] Wide Character length
      :wchar_t ** [out] ((|wcs|))
        output wchar
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
static
int
Xmbstowcs(const char * /* [in] */ mbstr,
          size_t * /* [in, out] */ sz,
          wchar_t ** /* [out] */ wcs) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (wcs && mbstr) {
        size_t dummy = 0;
        size_t wcs_len = -1;
        if (!sz) {
            sz = &dummy;
        }
        if (*sz) {
            mblen(NULL, 0); /* clear mbstate */
            wcs_len = mblen(mbstr, *sz);
        }
        else {
            wcs_len = mbstowcs(NULL, mbstr, 0);
        }
        if (wcs_len == (size_t)(-1)) {
            ret = OPENSOAP_INVALID_MB_SEQUENCE;
        }
        else {
            size_t ret_len = wcs_len + 1;
            *wcs = XwcsAlloc(ret_len);
            if (*wcs) {
                *sz = mbstowcs(*wcs, mbstr, ret_len);
                if (*sz == (size_t)(-1)) {
                    XwcsFree(wcs);
                    ret = OPENSOAP_INVALID_MB_SEQUENCE;
                }
                else {
                    ret = OPENSOAP_NO_ERROR;
                }
            }
            else {
                ret = OPENSOAP_MEM_BADALLOC;
            }
        }
    }
    
    return ret;
}

/*
=begin
--- function#Xwcsdup(wcs_str, wcs_len)

    :Parameters
      :const wchar_t * [in] ((|wcs_str|))
        Wide character string
      :size_t * [in, out] ((|wcs_len|))
        duplicated length. if *wcs_len > wcslen(wcs_str) then full copy.
    :Return Value
      :type
        wchar_t *
        wchar_t string
=end
 */
static
wchar_t *
Xwcsdup(const wchar_t * /* [in] */ wcs_str,
        size_t * /* [out] */ wcs_len) {
    wchar_t *ret = NULL;
    size_t dummy = 0;
    size_t wcs_str_len = wcslen(wcs_str);
    if (!wcs_len) {
        wcs_len = &dummy;
    }
    if (*wcs_len) {
        if (*wcs_len > wcs_str_len) {
            *wcs_len = wcs_str_len;
        }
        ret = XwcsAlloc(*wcs_len + 1);

        if (ret) {
            wcscpy(ret, wcs_str);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringFree( obj)

    :Parameters
      :OpenSOAPObjectPtr [in] ((|obj|))
    :Returns
      :int
	    Return Code Is ErrorCode
=end
 */
static
int
OpenSOAPStringFree(OpenSOAPObjectPtr obj) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (obj) {
        OpenSOAPStringPtr str = (OpenSOAPStringPtr)obj;
        XwcsFree(&str->stringEntity);
        ret = OpenSOAPObjectReleaseMembers(obj);
        if (OPENSOAP_SUCCEEDED(ret)) {
            free(str);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringCreate(str)
     0  OpenSOAP ̍쐬B
    
    :Parameters
       :OpenSOAPStringPtr * [out] ((|str|))
         쐬 OpenSOAP ̃|C^̊i[ꏊB
    :Return Value
	   :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCreate(OpenSOAPStringPtr * /* [out] */ str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        ret = OPENSOAP_MEM_BADALLOC;
        *str = malloc(sizeof(OpenSOAPString));
        if (*str) {
            ret = OpenSOAPObjectInitialize((OpenSOAPObjectPtr)*str,
                                           OpenSOAPStringFree);
            if (OPENSOAP_SUCCEEDED(ret)) {
                (*str)->stringEntity = NULL;
                (*str)->length = 0;
                ret = OpenSOAPStringClear(*str);
            }
            else {
                free(*str);
                *str = NULL;
            }
        }
    }

    return ret;
}
    
/*
=begin
--- function#OpenSOAPStringCreateWithMB(mb_str, str)
    }`oCgɂ菉 OpenSOAP ̍쐬B
    
    :Parameters
      :const char * [in] ((|mb_str|))
        ݒ肵}`oCgB
      :OpenSOAPStringPtr * [out] ((|str|))
        쐬 OpenSOAP ̃|C^̊i[ꏊB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCreateWithMB(const char * /* [in] */ mb_str,
                           OpenSOAPStringPtr * /* [out] */ str) {
    int ret = OpenSOAPStringCreate(str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = OpenSOAPStringSetStringMB(*str, mb_str);
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringCreateWithWC(wc_str, str)
    ChLN^ɂ菉 OpenSOAP ̍쐬B
    
    :Parameters
      :const wchar_t * [in] ((|wc_str|))
        ݒ肵ChLN^B
      :OpenSOAPStringPtr * [out] ((|str|))
        쐬 OpenSOAP ̃|C^̊i[ꏊB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCreateWithWC(const wchar_t * /* [in] */ wc_str,
                           OpenSOAPStringPtr * /* [out] */ str) {
    int ret = OpenSOAPStringCreate(str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = OpenSOAPStringSetStringWC(*str, wc_str);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringCreateWithCharEncodingString(char_enc, char_enc_str, str)
    Character-encoding w蕶ɂ菉 OpenSOAP ̍쐬B
    
    :Parameters
      :const char * [in] ((|char_enc|))
        w Character-encoding B
      :OpenSOAPByteArrayPtr [in] ((|char_enc_str|))
        ݒ肵f[^B
      :OpenSOAPStringPtr * [out] ((|str|))
        쐬 OpenSOAP ̃|C^̊i[ꏊB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCreateWithCharEncodingString(const char * /* [in] */ char_enc,
                                           OpenSOAPByteArrayPtr
                                           /* [in] */ char_enc_str,
                                           OpenSOAPStringPtr *
                                           /* [out] */ str) {
    int ret = OpenSOAPStringCreate(str);
    
    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = OpenSOAPStringSetCharEncodingString(*str,
                                                  char_enc,
                                                  char_enc_str);
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringRetain(str)
    QƂ̒ǉB
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringRetain(OpenSOAPStringPtr /* [in] */ str) {
    return OpenSOAPObjectRetain((OpenSOAPObjectPtr)str);
}

/*
=begin
--- function#OpenSOAPStringRelease(str)
    QƂ̊JBQƐ悪ȂȂꍇA\[X̊JsB
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringRelease(OpenSOAPStringPtr /* [in] */ str) {
    return OpenSOAPObjectRelease((OpenSOAPObjectPtr)str);
}

/*
=begin
--- function#OpenSOAPStringGetLengthMB(str, len)
    ݂ locale ɂ}`oCg̃TCY擾B
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :size_t * [out] ((|len|))
        TCYi[obt@ւ̃|C^B

    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetLengthMB(OpenSOAPStringPtr /* [in] */ str,
                          size_t * /* [out] */ len) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && len) {
        *len = (str->length) ? 
            wcstombs(NULL, str->stringEntity, 0) : 0;
        ret = (*len != (size_t)(-1)) ? OPENSOAP_NO_ERROR
            : OPENSOAP_INVALID_MB_SEQUENCE;
    }

    return ret;
}
        
/*
=begin
--- function#OpenSOAPStringGetLengthWC(str, len)
    ChLN^ɂ钷擾B
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :size_t * [out] ((|len|))
        i[obt@ւ̃|C^B

    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetLengthWC(OpenSOAPStringPtr /* [in] */ str,
                          size_t * /* [out] */ len) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && len) {
        *len = str->length;
        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}
        
/*
=begin
--- function#OpenSOAPStringGetStringMBAsByteArray(str, mb_str)
    ݂ locale ɂ}`oCg擾B
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :OpenSOAPByteArrayPtr [out] ((|mb_str|))
        i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
/* extern */
static
int
/* OPENSOAP_API */
OpenSOAPStringGetStringMBAsByteArray(OpenSOAPStringPtr /* [in] */ str,
                                     OpenSOAPByteArrayPtr /* [out] */ mb_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (mb_str) {
        size_t  mb_str_len = 0;
        ret = OpenSOAPStringGetLengthMB(str, &mb_str_len);
        if (OPENSOAP_SUCCEEDED(ret)) {
            if (!mb_str_len) {
                ret = OpenSOAPByteArrayClear(mb_str);
            }
            else {
                ret = OpenSOAPByteArrayResize(mb_str, mb_str_len + 1);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    unsigned char *mb_str_beg = NULL;
                    ret = OpenSOAPByteArrayBegin(mb_str, &mb_str_beg);
                    if (OPENSOAP_SUCCEEDED(ret)) {
                        mb_str_len = wcstombs(mb_str_beg,
                                              str->stringEntity,
                                              mb_str_len + 1);
                        ret = (mb_str_len == (size_t)(-1))
                            ? OPENSOAP_INVALID_MB_SEQUENCE :
                            OpenSOAPByteArrayResize(mb_str, mb_str_len);
                    }
                }
            }
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringGetStringMBWithAllocator(str, memAllocator, len, mbStr)
    OpenSOAP String GetStringMB with memAllocator

    :Parameters
      :[in]  OpenSOAPStringPtr ((|str|))
        OpenSOAP String Object pointer.
      :[in]  void * ( * ((|memAllocator|)) )(size_t)
        memAllocator function pointer. If NULL, then memAllocator is malloc.
      :[out] size_t * ((|len|))
	    length return buffer pointer. if NULL, then no effect.
      :[out] char ** ((|mbStr|))
	    MB string return buffer pointer. If NULL, then error.
		After this function call, You shuld release *mbStr's memory.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStringGetStringMBWithAllocator(/* [in]  */  OpenSOAPStringPtr str,
									   /* [in]  */  void *
									   (*memAllocator)(size_t),
									   /* [out] */ size_t *len,
									   /* [out] */ char **mbStr) {
	int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (mbStr) {
        OpenSOAPByteArrayPtr mbStrBa = NULL;
		*mbStr = NULL;
        ret = OpenSOAPByteArrayCreate(&mbStrBa);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringGetStringMBAsByteArray(str, mbStrBa);
            if (OPENSOAP_SUCCEEDED(ret)) {
                const unsigned char *mbStrBeg = NULL;
                size_t lenDummy = 0;
				if (!len) {
					len = &lenDummy;
				}
                ret = OpenSOAPByteArrayGetBeginSizeConst(mbStrBa,
                                                         &mbStrBeg,
                                                         len);
                if (OPENSOAP_SUCCEEDED(ret)) {
					if (!memAllocator) {
						memAllocator = malloc;
					}
					*mbStr = memAllocator(*len + 1);
                    if (*mbStr) {
                        memcpy(*mbStr, mbStrBeg, *len);
                        (*mbStr)[*len] = '\0';
                    }
                    else {
                        ret = OPENSOAP_MEM_BADALLOC;
                    }
                }
            }
            OpenSOAPByteArrayRelease(mbStrBa);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringGetStringWCWithAllocator(str, memAllocator, len, wcStr)
    OpenSOAP String GetStringWC with memAllocator

    :Parameters
      :[in]  OpenSOAPStringPtr ((|str|))
        OpenSOAP String Object pointer.
      :[in]  void * ( * ((|memAllocator|)) )(size_t)
        memAllocator function pointer. If NULL, then memAllocator is malloc.
      :[out] size_t * ((|len|))
	    length return buffer pointer. if NULL, then no effect.
      :[out] wchar_t ** ((|wcStr|))
	    WC string return buffer pointer. If NULL, then error.
		After this function call, You shuld release *wcStr's memory.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  allocater size_t parameter's unit is byte, not wchar_t.
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStringGetStringWCWithAllocator(/* [in]  */  OpenSOAPStringPtr str,
									   /* [in]  */  void *
									   (*memAllocator)(size_t),
									   /* [out] */ size_t *len,
									   /* [out] */ wchar_t **wcStr) {
	int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (wcStr) {
		if (!memAllocator) {
			memAllocator = malloc;
		}
		ret = OPENSOAP_MEM_BADALLOC;
		*wcStr = memAllocator(str->length * sizeof(wchar_t));
		if (*wcStr) {
			if (len) {
				*len = str->length;
			}
			wcscpy(*wcStr, str->stringEntity);
			ret = OPENSOAP_NO_ERROR;
		}
	}
	
    return ret;
}

/*
=begin
--- function#OpenSOAPStringGetStringMB(str, len, mb_str)
    ݂ locale ɂ}`oCg擾B
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :size_t * [in, out] ((|len|))
        TCYi[obt@ւ̃|C^B
        ďoO ((|mb_str|)) ̃TCYi[B
      :char * [out] ((|mb_str|))
        i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetStringMB(OpenSOAPStringPtr /* [in] */ str,
                          size_t * /* [in, out] */ len,
                          char * /* [out] */ mb_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (len && mb_str) {
        OpenSOAPByteArrayPtr mb_str_ba = NULL;
        ret = OpenSOAPByteArrayCreate(&mb_str_ba);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringGetStringMBAsByteArray(str, mb_str_ba);
            if (OPENSOAP_SUCCEEDED(ret)) {
                const unsigned char *mb_str_beg = NULL;
                size_t mb_str_len = 0;
                ret = OpenSOAPByteArrayGetBeginSizeConst(mb_str_ba,
                                                         &mb_str_beg,
                                                         &mb_str_len);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    if (*len < mb_str_len) {
                        ret = OPENSOAP_PARAMETER_BADVALUE;
                    }
                    else {
                        memcpy(mb_str, mb_str_beg, mb_str_len);
                        mb_str[mb_str_len] = '\0';
                        *len = mb_str_len;
                        ret = OPENSOAP_NO_ERROR;
                    }
                }
            }
            OpenSOAPByteArrayRelease(mb_str_ba);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringGetStringWC(str, len, mb_str)
    ChLN^擾B
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :size_t * [in, out] ((|len|))
        TCYi[obt@ւ̃|C^B
        ďoO ((|wc_str|)) ̒i[B
      :wchar_t * [out] ((|wc_str|))
        i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetStringWC(OpenSOAPStringPtr /* [in] */ str,
                          size_t * /* [in, out] */ len,
                          wchar_t * /* [out] */ wc_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && len && wc_str && (str->length < *len)) {
        wcscpy(wc_str, str->stringEntity);
        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}

/*
 */
static
const char *
GetDefaultCodeSet(void);

#define OpenSOAPStringIsDefaultCodeSet(charset) \
((charset) ? (strcmp((charset), GetDefaultCodeSet()) == 0) : 1)

/*
=begin
--- function#OpenSOAPStringGetCharEncodingString(str, char_enc, char_enc_str)
    Get character encoding string.
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :const char * [in] ((|char_enc|))
        character encoding.
      :OpenSOAPByteArrayPtr [out] ((|char_enc_str|))
        i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetCharEncodingString(OpenSOAPStringPtr /* [in] */ str,
                                    const char * /* [in] */ char_enc,
                                    OpenSOAPByteArrayPtr /* [out] */
                                    char_enc_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (char_enc_str) {
        if (OpenSOAPStringIsDefaultCodeSet(char_enc)) {
            ret = OpenSOAPStringGetStringMBAsByteArray(str,
                                                       char_enc_str);
        }
        else {
            OpenSOAPByteArrayPtr from_str = NULL;
            ret = OpenSOAPByteArrayCreate(&from_str);
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = OpenSOAPStringGetStringMBAsByteArray(str, from_str);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    ret = OpenSOAPStringConvertCharEncoding(NULL,
                                                            from_str,
                                                            char_enc,
                                                            char_enc_str);
            
                }
                OpenSOAPByteArrayRelease(from_str);
            }
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringSetStringMB(str, mb_str)
    ݂ locale ɂ}`oCgݒ肷B
    
    :Parameters
      :OpenSOAPStringPtr [in, out] ((|str|))
        OpenSOAP 
      :char * [in] ((|mb_str|))
        ݒ肵i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringSetStringMB(OpenSOAPStringPtr /* [in, out] */ str,
                          const char * /* [in] */ mb_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        if (mb_str && *mb_str) {
            size_t  mbs_len = 0;
            wchar_t *str_ent = NULL;
            ret = Xmbstowcs(mb_str, &mbs_len, &str_ent);
            if (OPENSOAP_SUCCEEDED(ret)) {
                str->length = mbs_len;
                XwcsFree(&(str->stringEntity));
                str->stringEntity = str_ent;
            }
        }
        else {
            ret = OpenSOAPStringClear(str);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringSetStringWC(str, mb_str)
    ChLN^ݒ肷B
    
    :Parameters
      :OpenSOAPStringPtr [in, out] ((|str|))
        OpenSOAP 
      :char * [in] ((|wc_str|))
        ݒ肵i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringSetStringWC(OpenSOAPStringPtr /* [in, out] */ str,
                          const wchar_t * /* [in] */ wc_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        if (wc_str && *wc_str) {
            size_t  wcs_len = -1;
            wchar_t *str_ent = Xwcsdup(wc_str, &wcs_len);
            if (str_ent) {
                str->length = wcs_len;
                XwcsFree(&(str->stringEntity));
                str->stringEntity = str_ent;
                ret = OPENSOAP_NO_ERROR;
            }
            else {
                ret = OPENSOAP_MEM_BADALLOC;
            }
        }
        else {
            ret = OpenSOAPStringClear(str);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringSetCharEncodingString(str, char_enc, char_enc_str)
    ݂ locale ɂ}`oCgݒ肷B
    
    :Parameters
      :OpenSOAPStringPtr [in, out] ((|str|))
        OpenSOAP 
      :const char * [in] ((|char_enc|))
        ݒ肵 Character Encoding.
      :OpenSOAPByteArrayPtr [in] ((|char_enc_str|))
        ݒ肵i[obt@ւ̃|C^B
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringSetCharEncodingString(OpenSOAPStringPtr /* [in, out] */ str,
                                    const char * /* [in] */ char_enc,
                                    OpenSOAPByteArrayPtr /* [in] */
                                    char_enc_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        OpenSOAPByteArrayPtr to_str = NULL;
        ret = OpenSOAPByteArrayCreate(&to_str);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringConvertCharEncoding(char_enc,
                                                    char_enc_str,
                                                    NULL,
                                                    to_str);
            
            if (OPENSOAP_SUCCEEDED(ret)) {
                /* null terminated */
                ret = OpenSOAPByteArrayAppend(to_str, NULL, 1);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    const unsigned char *to_str_beg = NULL;
                    ret = OpenSOAPByteArrayBeginConst(to_str, &to_str_beg);
                    if (OPENSOAP_SUCCEEDED(ret)) {
                        ret = OpenSOAPStringSetStringMB(str, to_str_beg);
                    }
                }
            }

            OpenSOAPByteArrayRelease(to_str);
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringClear(str)
    OpenSOAP NAāA 0 ɂB
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringClear(OpenSOAPStringPtr str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        wchar_t *cleared_str = XwcsRealloc(str->stringEntity, 1);

        if (cleared_str) {
            str->length = 0;
            cleared_str[0] = L'\0';
            str->stringEntity = cleared_str;
            ret = OPENSOAP_NO_ERROR;
        }
        else {
            ret = OPENSOAP_MEM_BADALLOC;
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringAppendMB(str, mb_str, mb_len)
    ݂ locale ɂ}`oCg OpenSOAP 
    ǉB
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :char * [in] ((|mb_str|))
        ǉi[obt@ւ̃|C^B
      :size_t [in] ((|mb_len|))
        ǉB0 ̏ꍇ́Amb_str SĂǉB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringAppendMB(OpenSOAPStringPtr str,
                       const char *mb_str,
                       size_t       mb_len) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        if (mb_str && *mb_str) {
            size_t add_mbs_len = 0;
            if (!mb_len) {
                mb_len = strlen(mb_str);
            }
            if (add_mbs_len != (size_t)(-1)) {
                wchar_t *wc_str = NULL;
                ret = Xmbstowcs(mb_str, &mb_len, &wc_str);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    ret = OpenSOAPStringAppendWC(str, wc_str, mb_len);
                    XwcsFree(&wc_str);
                }
            }
            else {
                ret = OPENSOAP_INVALID_MB_SEQUENCE;
            }
        }
        else {
            ret = OPENSOAP_NO_ERROR;
        }
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringAppendWC(str, wc_str, wc_len)
    ChLN^ OpenSOAP ɒǉB
    
    :Parameters
      :OpenSOAPStringPtr [in] ((|str|))
        OpenSOAP 
      :char * [in] ((|wc_str|))
        ǉi[obt@ւ̃|C^B
      :size_t [in] ((|wc_len|))
        ǉB0 ̏ꍇ́Amb_str SĂǉB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringAppendWC(OpenSOAPStringPtr str,
                       const wchar_t *wc_str,
                       size_t   wc_len) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str) {
        if (wc_str && *wc_str) {
            size_t new_len = str->length;
            wchar_t *new_str = NULL;
            size_t add_len = wcslen(wc_str);

            ret = OPENSOAP_NO_ERROR;
            if (!wc_len) {
                wc_len = add_len;
            }
            if (wc_len < add_len) {
                add_len = wc_len;
            }
            new_len += add_len;
            new_str = XwcsRealloc(str->stringEntity, new_len + 1);
            if (!new_str) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                wcsncat(new_str, wc_str, add_len);
                str->stringEntity = new_str;
                str->length = new_len;
            }
        }
    }

    return ret;
}

#if defined(HAVE_VSWPRINTF)    
/*
=begin
--- function#OpenSOAPStringVFormatWC(str, format, ap)
    `B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const wchar_t * [in] ((|format|))
            format B
        :va_list [in] ((|ap|))
            ψB
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
/* OPENSOAP_API */
OpenSOAPStringVFormatWC(OpenSOAPStringPtr /* [in] */ str,
                       const wchar_t * /* [in] */ format,
                       va_list ap) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && format) {
#define BUFGRAWSIZE 256
        size_t  maxlen = wcslen(format) + BUFGRAWSIZE;
        int wcs_len = 0;
        wchar_t *wcs_buf = NULL;
        wchar_t *old_wcs_buf = NULL;

        for (; wcs_buf = XwcsRealloc(old_wcs_buf, (maxlen + 1));
             old_wcs_buf = wcs_buf) {
            wcs_len = vswprintf(wcs_buf, maxlen, format, ap);
            if (wcs_len > -1) {
                break;
            }
            maxlen += BUFGRAWSIZE;
        }
        if (!wcs_buf) {
            XwcsFree(&old_wcs_buf);
            ret = OPENSOAP_MEM_BADALLOC;
        }
        else {
            if (!wcs_len) {
                XwcsFree(&wcs_buf);
            }
            XwcsFree(&str->stringEntity);
            str->stringEntity = wcs_buf;
            str->length = wcs_len;

            ret = OPENSOAP_NO_ERROR;
        }
#undef BUFGRAWSIZE
    }

    return ret;
}
#else /* !HAVE_VSWPRINTF */
/*
=begin
--- function#OpenSOAPStringVFormatMB(str, format, ap)
    `B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const char * [in] ((|format|))
            format B
        :va_list [in] ((|ap|))
            ψB
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
/* OPENSOAP_API */
OpenSOAPStringVFormatMB(OpenSOAPStringPtr /* [in] */ str,
                        const char * /* [in] */ format,
                        va_list ap) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && format) {
#define BUFGRAWSIZE 256
        size_t  maxlen = strlen(format) + BUFGRAWSIZE;
        int len = 0;
        char *buf = NULL;
        char *old_buf = NULL;

        for (; buf = realloc(old_buf, (maxlen + 1));
             old_buf = buf) {
            len = vsnprintf(buf, maxlen, format, ap);
            if (len > -1) {
                break;
            }
            maxlen += BUFGRAWSIZE;
        }
        if (!buf) {
            free(old_buf);
            ret = OPENSOAP_MEM_BADALLOC;
        }
        else {
            ret = OpenSOAPStringSetStringMB(str, buf);
            free(buf);
        }
#undef BUFGRAWSIZE
    }

    return ret;
}

#endif /* HAVE_VSWPRINTF */

/*
=begin
--- function#OpenSOAPStringFormatMB(str, format, ...)
    `B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const char * [in] ((|format|))
            format B
        :...
            ψB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFormatMB(OpenSOAPStringPtr /* [in] */ str,
                       const char * /* [in] */ format, ...) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    va_list ap;
#if defined(HAVE_VSWPRINTF)
    wchar_t *format_wstr = NULL;

    ret = Xmbstowcs(format, NULL, &format_wstr);
    if (OPENSOAP_SUCCEEDED(ret)) {
        va_start(ap, format);
        ret = OpenSOAPStringVFormatWC(str, format_wstr, ap);
        va_end(ap);
        XwcsFree(&format_wstr);
    }
#else /* !HAVE_VSWPRINTF */
    va_start(ap, format);
    ret = OpenSOAPStringVFormatMB(str, format, ap);
    va_end(ap);
#endif /* !HAVE_VSWPRINTF */
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringFormatWC(str, format, ...)
    `B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const wchar_t * [in] ((|format|))
            format B
        :...
            ψB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFormatWC(OpenSOAPStringPtr /* [in] */ str,
                       const wchar_t * /* [in] */ format, ...) {
    int ret = OPENSOAP_NOT_CATEGORIZE_ERROR;
    va_list ap;

#if defined(HAVE_VSWPRINTF)
    va_start(ap, format);
    ret = OpenSOAPStringVFormatWC(str, format, ap);
    va_end(ap);
#else /* !HAVE_VSWPRINTF */
    OpenSOAPStringPtr format_str = NULL;
    ret = OpenSOAPStringCreateWithWC(format, &format_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        size_t format_mb_sz = 0;
        ret = OpenSOAPStringGetLengthMB(format_str, &format_mb_sz);
        if (OPENSOAP_SUCCEEDED(ret)) {
            char *format_mb = malloc(format_mb_sz + 1);
            if (!format_mb) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                ret = OpenSOAPStringGetStringMB(format_str,
                                                &format_mb_sz,
                                                format_mb);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    va_start(ap, format);
                    ret = OpenSOAPStringVFormatMB(str, format_mb, ap);
                    va_end(ap);
                }

                free(format_mb);
            }
        }
        OpenSOAPStringRelease(format_str);
    }
#endif /* !HAVE_VSWPRINTF */
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringCompareMB(str, cmp_str, cmp_rslt)
    rB

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const char * [in] ((|cmp_str|))
            rB
        :int * [out] ((|cmp_rslt|))
            rʁB*cmp_rslt ̒l̈Ӗ́Astrcmp ƓB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCompareMB(OpenSOAPStringPtr /* [in] */ str,
                        const char * /* [in] */ cmp_str,
                        int * /* [out] */ cmp_rslt) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (cmp_str) {
        wchar_t *cmp_wstr = NULL;
        ret = Xmbstowcs(cmp_str, NULL, &cmp_wstr);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringCompareWC(str, cmp_wstr, cmp_rslt);
            XwcsFree(&cmp_wstr);
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringCompareWC(str, cmp_str, cmp_rslt)
    rB

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const wchar_t * [in] ((|cmp_str|))
            rB
        :int * [out] ((|cmp_rslt|))
            rʁB*cmp_rslt ̒l̈Ӗ́Astrcmp ƓB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCompareWC(OpenSOAPStringPtr /* [in] */ str,
                        const wchar_t * /* [in] */ cmp_str,
                        int * /* [out] */ cmp_rslt) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && cmp_str && cmp_rslt) {
        *cmp_rslt = wcscmp(str->stringEntity, cmp_str);
        ret = OPENSOAP_NO_ERROR;
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringCompare(str, cmp_str, cmp_rslt)
    rB

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :OpenSOAPStringPtr [in] ((|cmp_str|))
            rB
        :int * [out] ((|cmp_rslt|))
            rʁB*cmp_rslt ̒l̈Ӗ́Astrcmp ƓB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringCompare(OpenSOAPStringPtr /* [in] */ str,
                      OpenSOAPStringPtr /* [in] */ cmp_str,
                      int * /* [out] */ cmp_rslt) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (cmp_str) {
        ret = OpenSOAPStringCompareWC(str,
                                      cmp_str->stringEntity,
                                      cmp_rslt);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringFindStringMB(str, find_str, idx)
    񌟍B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const char * [in] ((|find_str|))
            B
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] ʃCfbNXB񂪑݂Ȃꍇ
            ͒l (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFindStringMB(OpenSOAPStringPtr /* [in] */ str,
                           const char * /* [in] */ find_str,
                           size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (find_str) {
        wchar_t *find_wstr = NULL;
        ret = Xmbstowcs(find_str, NULL, &find_wstr);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringFindStringWC(str, find_wstr, idx);
            XwcsFree(&find_wstr);
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringFindStringWC(str, find_str, idx)
    񌟍B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :const wchar_t * [in] ((|find_str|))
            B
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] ʃCfbNXB񂪑݂Ȃꍇ
            ͒l (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFindStringWC(OpenSOAPStringPtr /* [in] */ str,
                           const wchar_t * /* [in] */ find_str,
                           size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && find_str && idx &&
        (*idx != (size_t)(-1))) {
        size_t find_idx = *idx;
        *idx = (size_t)(-1);
        if  (find_idx < str->length) {
            const wchar_t *findstr_loc
                = wcsstr(str->stringEntity + find_idx, find_str);
            if (findstr_loc) {
                *idx = (findstr_loc - str->stringEntity);
            }
        }
        ret = OPENSOAP_NO_ERROR;
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringFindString(str, find_str, idx)
    񌟍B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :OpenSOAPStringPtr [in] ((|find_str|))
            B
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] ʃCfbNXB񂪑݂Ȃꍇ
            ͒l (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFindString(OpenSOAPStringPtr /* [in] */ str,
                         OpenSOAPStringPtr /* [in] */ find_str,
                         size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (find_str) {
        ret = OpenSOAPStringFindStringWC(str,
                                         find_str->stringEntity,
                                         idx);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringFindIfStringIndex(str, find_func, find_func_opt, idx)
    񌟍B

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
          OpenSOAP String
        :OpenSOAPStringFindIfFunc [in] ((|find_func|))
          ֐B
        :void * [in] ((|find_func_opt|))
          ֐B
        :size_t * [in, out] ((|idx|))
          [in]  JnCfbNXB
          [out] ʃCfbNXB񂪑݂Ȃꍇ
          ͒l (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringFindIfStringIndex(OpenSOAPStringPtr /* [in] */ str,
                                OpenSOAPStringFindIfFunc /* [in] */ find_func,
                                void * /* [in] */ find_func_opt,
                                size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && find_func && idx &&
        (*idx != (size_t)(-1))) {
        size_t find_idx = *idx;
        *idx = (size_t)(-1);
        for (;find_idx < str->length; ++find_idx) {
            int judge = 0;
            ret = find_func(find_func_opt,
                            str->stringEntity[find_idx],
                            &judge);
            if (OPENSOAP_FAILED(ret) || judge) {
                break;
            }
        }
        if (OPENSOAP_SUCCEEDED(ret)) {
            *idx = find_idx;
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringReplaceStringMB(str, find_str, rplc_str, idx)
    uB

    :Parameters
        :OpenSOAPStringPtr [in, out] ((|str|))
            OpenSOAP String
        :const char * [in] ((|find_str|))
            B
        :const char * [in] ((|rplc_str|))
            uB
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] uJnCfbNXBułȂꍇ͒l
            (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringReplaceStringMB(OpenSOAPStringPtr /* [in, out] */ str,
                              const char * /* [in] */ find_str,
                              const char * /* [in] */ rplc_str,
                              size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (find_str && rplc_str) {
        wchar_t *find_wstr = NULL;
        ret = Xmbstowcs(find_str, NULL, &find_wstr);
        if (OPENSOAP_SUCCEEDED(ret)) {
            wchar_t *rplc_wstr = NULL;
            ret = Xmbstowcs(rplc_str, NULL, &rplc_wstr);
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = OpenSOAPStringReplaceStringWC(str,
                                                    find_wstr,
                                                    rplc_wstr,
                                                    idx);
                XwcsFree(&rplc_wstr);
            }
            XwcsFree(&find_wstr);
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringReplaceStringWC(str, find_str, rplc_str, idx)
    uB

    :Parameters
        :OpenSOAPStringPtr [in, out] ((|str|))
            OpenSOAP String
        :const wchar_t * [in] ((|find_str|))
            B
        :const wchar_t * [in] ((|rplc_str|))
            uB
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] uJnCfbNXBułȂꍇ͒l
            (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringReplaceStringWC(OpenSOAPStringPtr /* [in, out] */ str,
                              const wchar_t * /* [in] */ find_str,
                              const wchar_t * /* [in] */ rplc_str,
                              size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (rplc_str) {
        ret = OpenSOAPStringFindStringWC(str, find_str, idx);
        if (OPENSOAP_SUCCEEDED(ret)) {
            size_t find_str_len = wcslen(find_str);
            size_t rplc_str_len = wcslen(rplc_str);
            size_t replaced_len = str->length + rplc_str_len - find_str_len;
            wchar_t *replaced_str = str->stringEntity;

            if (replaced_len != str->length) {
                if (replaced_len > str->length) {
                    replaced_str = XwcsRealloc(replaced_str,
                                               (replaced_len + 1));
                }
                if (replaced_str) {
                    wmemmove(replaced_str + *idx + rplc_str_len,
                             replaced_str + *idx + find_str_len,
                             str->length - *idx - find_str_len + 1);
                    
                    if (replaced_len < str->length) {
                        replaced_str = XwcsRealloc(replaced_str,
                                                   (replaced_len + 1));
                    }
                }
            }
            if (!replaced_str) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                wmemcpy(replaced_str + *idx,
                        rplc_str,
                        rplc_str_len);

                str->length = replaced_len;
                str->stringEntity = replaced_str;
            }
        }
    }
    
    return ret;
}

/*
=begin
--- function#OpenSOAPStringReplaceString(str, find_str, rplc_str, idx)
    uB

    :Parameters
        :OpenSOAPStringPtr [in, out] ((|str|))
            OpenSOAP String
        :OpenSOAPStringPtr [in] ((|find_str|))
            B
        :OpenSOAPStringPtr [in] ((|rplc_str|))
            uB
        :size_t * [in, out] ((|idx|))
            [in]  JnCfbNXB
            [out] uJnCfbNXBułȂꍇ͒l
            (size_t)(-1) ƂȂB
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringReplaceString(OpenSOAPStringPtr /* [in, out] */ str,
                            OpenSOAPStringPtr /* [in] */ find_str,
                            OpenSOAPStringPtr /* [in] */ rplc_str,
                            size_t * /* [in, out] */ idx) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (find_str) {
        ret = OpenSOAPStringReplaceStringWC(str,
                                            find_str->stringEntity,
                                            rplc_str->stringEntity,
                                            idx);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringDuplicate(str, dup_str)
    Duplicate string.

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
            OpenSOAP String
        :OpenSOAPStringPtr * [out] ((|dup_str|))
            duplicate result.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringDuplicate(OpenSOAPStringPtr /* [in] */ str,
                        OpenSOAPStringPtr * /* [out] */ dup_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && dup_str && (*dup_str != str)) {
        ret = (*dup_str) ?
            OpenSOAPStringSetStringWC(*dup_str,
                                      str->stringEntity)
            : OpenSOAPStringCreateWithWC(str->stringEntity,
                                         dup_str);
    }

    return ret;
}

/*
=begin
--- function#OpenSOAPStringGetSubstring(str, beg, len, sub_str)
    Duplicate string.

    :Parameters
        :OpenSOAPStringPtr [in] ((|str|))
          OpenSOAP String
        :size_t [in] ((|beg|))
          substring begin index.
        :size_t [in] ((|len|))
          substring length. if len == -1 then copy to end of string.
        :OpenSOAPStringPtr * [out] ((|sub_str|))
          substring
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
*/
extern
int
OPENSOAP_API
OpenSOAPStringGetSubstring(OpenSOAPStringPtr /* [in] */ str,
                           size_t   /* [in] */ beg,
                           size_t   /* [in] */ len,
                           OpenSOAPStringPtr * /* [out] */ sub_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (str && sub_str && len && beg > str->length) {
        int is_create = !*sub_str;
        if (is_create) {
            ret = OpenSOAPStringCreate(sub_str);
        }
        if (!is_create || OPENSOAP_SUCCEEDED(ret)) {
            wchar_t *sub_beg = str->stringEntity + beg;
            size_t  sub_len = len;
            wchar_t *str_ent = Xwcsdup(sub_beg, &sub_len);
            if (str_ent) {
                (*sub_str)->length = sub_len;
                XwcsFree(&((*sub_str)->stringEntity));
                (*sub_str)->stringEntity = str_ent;
                ret = OPENSOAP_NO_ERROR;
            }
            else {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            if (is_create && OPENSOAP_FAILED(ret)) {
                OpenSOAPStringRelease(*sub_str);
            }
        }
    }

    return ret;
}

#if defined(NONGCC_WIN32)
#include <windows.h>

/*
=begin
= convert ACP to Codeset
=end
*/
typedef struct {
    UINT    acp;
    const char *codeset;
} ACPCodeSetMapItem;

ACPCodeSetMapItem ACPCodeSetMap[] = {
    {0, "US-ASCII"}, /* ACP is wrong */
    {932, "Shift_JIS"},
    {0, NULL}
};

/*
=begin
--- function#GetDefaultCodeSet()
    
    :Parameters
    :Return Value
      :const char *
	:Note
	  This Function Is Static Function
=end
*/
static
const char *
GetDefaultCodeSet(void) {
    UINT acp = GetACP();
    const ACPCodeSetMapItem *c_map_i = ACPCodeSetMap;
    const char *ret = c_map_i->codeset;

    for (; c_map_i->codeset; ++c_map_i) {
        if  (acp == c_map_i->acp) {
            ret = c_map_i->codeset;
            break;
        }
    }

    return ret;
}

#else /* !NONGCC_WIN32 */
#if defined(HAVE_LANGINFO_H)
#include <langinfo.h>
#endif /* HAVE_LANGINFO_H */

#if defined(CODESET) && defined(HAVE_LANGINFO_H)

const char *
defaultCodesets[] = {
	"ANSI_X3.4-1968",
	"646",
	NULL
};

static
const char *
GetDefaultCodeSet(void) {
	static const char defaultCodeset[]	= "US-ASCII";
    const char *ret = nl_langinfo(CODESET);
	const char **defCodeset = defaultCodesets;

	if (!ret) {
		ret = defaultCodeset;
	}
	for (; *defCodeset; ++defCodeset) {
		if (strcmp(ret, *defCodeset) == 0) {
			ret = defaultCodeset;
			break;
		}
	}

    return ret;
}
#else /* !CODESET || !HAVE_LANGINFO_H */

typedef struct {
    const char *lc_ctype;
    const char *codeset;
} LC_CTYPECodeSetMapItem;

static
const LC_CTYPECodeSetMapItem
LC_CTYPECodeSetMap[] = {
    {"ja_JP.EUC-JP", "EUC-JP"},
    {"ja_JP.eucJP", "EUC-JP"},
    {"ja_JP.ujis", "EUC-JP"},
    {"ja_JP", "EUC-JP"},
    {"japanese.euc", "EUC-JP"},
    {"japanese", "EUC-JP"},
    {"ja_JP.SJIS", "Shift_JIS"},
    {"japanese.sjis", "Shift_JIS"},
    {"ja_JP.UTF-8", "UTF-8"},
    {"C", "US-ASCII"},
    {NULL, "US-ASCII"}
};
/*
  Following function is wrong if setlocale call with second parameter is not ""
 */
static
const char *
GetDefaultCodeSet(void) {
    const LC_CTYPECodeSetMapItem *map_i = LC_CTYPECodeSetMap;
    const char *LC_CTYPE_ENV = getenv("LC_ALL");
    if (!LC_CTYPE_ENV) {
        LC_CTYPE_ENV = getenv("LC_CTYPE");
        if (!LC_CTYPE_ENV) {
            LC_CTYPE_ENV = getenv("LANG");
        }
    }
    for (; map_i->lc_ctype; ++map_i) {
        if (strcmp(map_i->lc_ctype, LC_CTYPE_ENV) == 0) {
            break;
        }
    }
    
    return map_i->codeset;
}
#endif /* !CODESET */

#endif /* !NONGCC_WIN32 */

#if defined(HAVE_ICONV_H)
#include <locale.h>
#include <iconv.h>

#if defined(__linux__)
/*
  Why second arg type of iconv declare in iconv.h is char ** at my environment?
 */
typedef char * ICONV_SECOND_ARG_TYPE_REF;
#else  /* !__linux__ */
typedef const char *ICONV_SECOND_ARG_TYPE_REF;
#endif /* !__linux__ */

/*
=begin
--- function#OpenSOAPStringConvertCharEncodingWithSize(from_enc, from_str, to_enc, to_str)
    static function.
    Convert character encoding.
    This function isn't include OpenSOAPString member,
    but this has something to do with String processing.

    :Parameters
      :const char * [in] ((|from_enc|))
        Convert source character encoding. If NULL or "" then current
        locale encoding.
      :const unsigned char * [in] ((|from_beg|))
        Convert source string data.
      :const unsigned char * [in] ((|from_sz|))
        Convert source string data size.
      :const char * [in] ((|to_enc|))
        Convert distination character encoding. If NULL or "" then current
        locale encoding.
      :OpenSOAPByteArrayPtr [out] ((|to_str|))
        Convert distination string data. If from_enc equal to to_enc,
        then copy from_str to to_str.
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
 */
static
int
OpenSOAPStringConvertCharEncodingWithSize(const char * /* [in] */ from_enc,
                                          const unsigned char *
                                          /* [in] */ from_beg,
                                          size_t /* [in] */ from_sz,
                                          const char * /* [in] */ to_enc,
                                          OpenSOAPByteArrayPtr
                                          /* [out] */ to_str) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
    if (!from_sz) {
        ret = OpenSOAPByteArrayClear(to_str);
    }
    else {
        const char *def_enc = GetDefaultCodeSet();
    
        if (!from_enc || !(*from_enc)) {
            from_enc = def_enc;
        }
        if (!to_enc || !(*to_enc)) {
            to_enc = def_enc;
        }
    
        if (strcmp(from_enc, to_enc) == 0) {
            /* duplicate */
            ret = OpenSOAPByteArrayClear(to_str);
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = OpenSOAPByteArrayAppend(to_str,
                                              from_beg,
                                              from_sz);
            }
        }
        else {
            iconv_t cd = iconv_open(to_enc, from_enc);
            
            if (cd == (iconv_t)(-1)) {
                ret = OPENSOAP_ICONV_NOT_IMPL;
            }
            else {
                ICONV_SECOND_ARG_TYPE_REF from_buf
                    = (ICONV_SECOND_ARG_TYPE_REF)from_beg;

				/* clear to_str */
				ret = OpenSOAPByteArrayClear(to_str);
                while (from_sz && OPENSOAP_SUCCEEDED(ret)) {
#define TMP_BUF_SZ  256
                    char    tmp_buf[TMP_BUF_SZ];
                    char    *to_buf = tmp_buf;
                    size_t  to_left = TMP_BUF_SZ;
                    size_t  iconv_result = (size_t)(-1);

                    while ((size_t)(-1) !=
                           (iconv_result = iconv(cd,
                                                 &from_buf, &from_sz,
                                                 &to_buf, &to_left))
                           && from_sz) {
                    }

                    if (iconv_result == (size_t)(-1)) {
                        if (errno == EILSEQ) {
                            ret = OPENSOAP_INVALID_MB_SEQUENCE;
                            break;
                        }
                        else if (errno == EINVAL) {
                            ret = OPENSOAP_INVALID_MB_SEQUENCE;
                            break;
                        }
                    }
                    ret = OpenSOAPByteArrayAppend(to_str,
                                                  tmp_buf,
                                                  to_buf - tmp_buf);
#define TMP_BUF_SZ  256
                }
            
                iconv_close(cd);
            }
        }
    }
    
    return ret;
}

#else  /* !HAVE_ICONV_H */
/*
  Inchiki charencoding converter
*/
# if defined(NONGCC_WIN32)
/* for Win32 API's implement */
typedef wchar_t unichar_t;
# else /* !NONGCC_WIN32 */
/* for having no iconv no Win32 */
typedef unsigned short unichar_t;
# endif /* !NONGCC_WIN32 */

/* OpenSOAPUnicharArray */
typedef OpenSOAPByteArrayPtr OpenSOAPUnicharArrayPtr;

#define OpenSOAPUnicharArrayCreate(u_ary) OpenSOAPByteArrayCreate(u_ary)
#define OpenSOAPUnicharArrayRelease(u_ary) OpenSOAPByteArrayRelease(u_ary)
#define OpenSOAPUnicharArrayClear(u_ary) OpenSOAPByteArrayClear(u_ary)
#define OpenSOAPUnicharArrayResize(u_ary, sz) \
OpenSOAPByteArrayResize(u_ary, sz)
#define OpenSOAPUnicharArrayAppend(u_ary, data, sz) \
OpenSOAPByteArrayAppend(u_ary, \
(const unsigned char *)(data), (sz) * sizeof(unichar_t))
#define OpenSOAPUnicharArraySetData(u_ary, data, sz) \
OpenSOAPByteArraySetData(u_ary, \
(const unsigned char *)(data), (sz) * sizeof(unichar_t))
#define OpenSOAPUnicharArrayBegin(u_ary, beg) \
OpenSOAPByteArrayBegin(u_ary, (unsigned char **)(beg))

/*
=begin
--- function#OpenSOAPUnicharArrayGetBeginSizeConst( u_ary, beg, sz) 

    :Parameters
        :OpenSOAPUnicharArrayPtr [in] ((|u_ary|))
        :const unichar_t ** [out] ((|beg|))
        :size_t * [out] ((|sz|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
OpenSOAPUnicharArrayGetBeginSizeConst(OpenSOAPUnicharArrayPtr /* [in] */ u_ary,
                                      const unichar_t ** /* [out] */ beg,
                                      size_t * /* [out] */ sz) {
    int ret
        = OpenSOAPByteArrayGetBeginSizeConst(u_ary,
                                             (const unsigned char **)(beg),
                                             sz);
    if (OPENSOAP_SUCCEEDED(ret)) {
        *sz /= 2;
    }

    return ret;
}

typedef
int
(*FromUnicharFunc)(const unichar_t * /* [in] */ from_str,
                   size_t /* [in] */ from_len,
                   OpenSOAPByteArrayPtr /* [out] */ to_str);

typedef
int
(*ToUnicharFunc)(const unsigned char * /* [in] */ from_str,
                 size_t /* [in] */ from_sz,
                 OpenSOAPUnicharArrayPtr /* [out] */ to_str);

typedef
int
(*MBConvFunc)(const unsigned char * /* [in] */ from_str,
              size_t /* [in] */ from_sz,
              OpenSOAPByteArrayPtr /* [out] */ to_str);


/*---------------------------------------------------*/
#if 0
/* template */
/*
=begin
= Unicode
=end
*/

/*
=begin
--- function#UnicharTo( from_str, from_len, to_str)

    :Parameters
        :const unichar_t * [in] ((|from_str|))
        :size_t [in] ((|from_len|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharTo(const unichar_t * /* [in] */ from_str,
          size_t /* [in] */ from_len,
          OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
    }

    return ret;
}

/*
=begin
--- function#ToUnichar( from_str, from_sz, to_str)

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPUnicharArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
ToUnichar(const unsigned char * /* [in] */ from_str,
          size_t /* [in] */ from_sz,
          OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPUnicharArrayClear(to_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
    }

    return ret;
}

#endif
/*
=begin
= non Unicode(USASCII)
=end
*/

/*
=begin
--- function#USASCIIToUSASCII( from_str, from_sz, to_str)
    to US-ASCII if input not US-ASCII then error

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
USASCIIToUSASCII(const unsigned char * /* [in] */from_str,
                 size_t /* [in] */ from_sz,
                 OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayResize(to_str, from_sz);
    
    if (OPENSOAP_SUCCEEDED(ret)) {
        unsigned char *to_i = NULL;
        ret = OpenSOAPByteArrayBegin(to_str, &to_i);
        if (OPENSOAP_SUCCEEDED(ret)) {
            const unsigned char *c_end = from_str + from_sz;
            const unsigned char *c_i = from_str;
            for (; c_i != c_end && OPENSOAP_SUCCEEDED(ret);
                 ++c_i, ++to_i) {
                if (*c_i > 0x7f) {
                    ret = OPENSOAP_INVALID_MB_SEQUENCE;
                }
                else {
                    *to_i = *c_i;
                }
            }
        }
    }

    return ret;
}

/*
=begin
--- function#UnicharToUSASCII( from_str, from_len, to_str)
    unichar_t -> US-ASCII

    :Parameters
        :const unichar_t * [in] ((|from_str|))
        :size_t [in] ((|from_len|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToUSASCII(const unichar_t * /* [in] */ from_str,
                 size_t /* [in] */ from_len,
                 OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (from_str && from_len) {
        /* clear */
        ret = OpenSOAPByteArrayClear(to_str);
        if (OPENSOAP_SUCCEEDED(ret)) {
            unsigned char *buf = malloc(from_len);
            if (!buf) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                const unichar_t *w_end = from_str + from_len;
                const unichar_t *w = from_str;
                unsigned char *to = buf;

                for (; w != w_end; *to++ = (unsigned char)*w++) {
                    if (*w > 0x7f) {
                        ret = OPENSOAP_INVALID_MB_SEQUENCE;
                        break;
                    }
                }

                if (w == w_end) {
                    ret = OpenSOAPByteArrayAppend(to_str,
                                                  buf,
                                                  from_len);
                }

                free(buf);
            }
        }
    }

    return ret;
}

/*
=begin
--- function#USASCIIToUnichar( from_str, from_sz, to_str)

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPUnicharArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
USASCIIToUnichar(const unsigned char * /* [in] */ from_str,
                 size_t /* [in] */ from_sz,
                 OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    
    if (from_str && from_sz) {
        /* clear */
        ret = OpenSOAPUnicharArrayClear(to_str);
        if (OPENSOAP_SUCCEEDED(ret)) {
            const unsigned char *c_end = from_str + from_sz;
            const unsigned char *c = from_str;
            for (; c != c_end && OPENSOAP_SUCCEEDED(ret); ++c) {
                unichar_t u_c = *c;
                ret = OpenSOAPUnicharArrayAppend(to_str,
                                                 &u_c, 1);
            }
        }
    }

    return ret;
}

/*
=begin
--- function#UnicharToLatin1(fromStr, fromLen, toStr)
    unichar_t -> ISO 8859-1

    :Parameters
        :[in]  const unichar_t * ((|from_str|))
        :[in]  size_t ((|from_len|))
        :[out] OpenSOAPByteArrayPtr ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToLatin1(/* [in]  */ const unichar_t *fromStr,
                /* [in]  */ size_t fromLen,
                /* [out] */ OpenSOAPByteArrayPtr toStr) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (fromStr && fromLen) {
        /* clear */
        ret = OpenSOAPByteArrayClear(toStr);
        if (OPENSOAP_SUCCEEDED(ret)) {
            unsigned char *buf = malloc(fromLen);
            if (!buf) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                const unichar_t *wEnd = fromStr + fromLen;
                const unichar_t *w = fromStr;
                unsigned char *to = buf;

                for (; w != wEnd; *to++ = (unsigned char)*w++) {
                    if (*w > 0xff) {
                        ret = OPENSOAP_INVALID_MB_SEQUENCE;
                        break;
                    }
                }

                if (w == wEnd) {
                    ret = OpenSOAPByteArrayAppend(toStr,
                                                  buf,
                                                  fromLen);
                }

                free(buf);
            }
        }
    }

    return ret;
}

/*
=begin
--- function#Latin1ToUnichar(fromStr, fromSz, toStr)

    :Parameters
        :[in]  const unsigned char * ((|from_str|))
        :[in]  size_t ((|from_sz|))
        :[out] OpenSOAPUnicharArrayPtr ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
Latin1ToUnichar(/* [in]  */ const unsigned char *fromStr,
                /* [in]  */ size_t fromSz,
                /* [out] */ OpenSOAPUnicharArrayPtr toStr) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    
    if (fromStr && fromSz) {
        /* clear */
        ret = OpenSOAPUnicharArrayClear(toStr);
        if (OPENSOAP_SUCCEEDED(ret)) {
            const unsigned char *cEnd = fromStr + fromSz;
            const unsigned char *c = fromStr;
            for (; c != cEnd && OPENSOAP_SUCCEEDED(ret); ++c) {
                unichar_t uChar = *c;
                ret = OpenSOAPUnicharArrayAppend(toStr,
                                                 &uChar, 1);
            }
        }
    }

    return ret;
}

# if defined(NONGCC_WIN32)
/*
=begin
= CP932 (Shift_JIS)
=end
*/
/*
=begin
--- function#UnicharToCP932( from_str, from_len, to_str)

    :Parameters
        :const unichar_t * [in] ((|from_str|))
        :size_t [in] ((|from_len|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToCP932(const unichar_t * /* [in] */ from_str,
               size_t /* [in] */ from_len,
               OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    if (!from_len) {
        ret = OpenSOAPByteArrayClear(to_str);
    }
    else {
        const UINT encodingCP = 932;
        BOOL    is_used_def_char = FALSE;
        int mb_len
            = WideCharToMultiByte(encodingCP,
                                  WC_COMPOSITECHECK,
                                  from_str,
                                  from_len,
                                  NULL,
                                  0,
                                  NULL,
                                  &is_used_def_char);
    
        if (is_used_def_char) {
            ret = OPENSOAP_INVALID_MB_SEQUENCE;
        }
        else if (!mb_len) {
            DWORD last_error = GetLastError();
            ret = (last_error == ERROR_INSUFFICIENT_BUFFER)
                ? OPENSOAP_MEM_OUTOFRANGE
                : OPENSOAP_PARAMETER_BADVALUE;
        }
        else {
            ret = OpenSOAPByteArrayResize(to_str, (size_t)mb_len);
            if (OPENSOAP_SUCCEEDED(ret)) {
                unsigned char *to_str_beg = NULL;
                size_t to_str_sz = 0;
                ret = OpenSOAPByteArrayGetBeginSize(to_str,
                                                    &to_str_beg,
                                                    &to_str_sz);
                if (OPENSOAP_SUCCEEDED(ret)) {
                    mb_len 
                        = WideCharToMultiByte(encodingCP,
                                              WC_COMPOSITECHECK,
                                              from_str,
                                              from_len,
                                              to_str_beg,
                                              to_str_sz,
                                              NULL,
                                              &is_used_def_char);
                        
                    if (is_used_def_char) {
                        ret = OPENSOAP_INVALID_MB_SEQUENCE;
                    }
                    else if (!mb_len) {
                        DWORD last_error = GetLastError();
                        ret = (last_error == ERROR_INSUFFICIENT_BUFFER)
                            ? OPENSOAP_MEM_OUTOFRANGE
                            : OPENSOAP_PARAMETER_BADVALUE;
                    }
                }
            }
        }
    }
    
    return ret;
}

/*
=begin
--- function#CP932ToUnichar( from_str, from_sz, to_str)

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPUnicharArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
CP932ToUnichar(const unsigned char * /* [in] */ from_str,
               size_t /* [in] */ from_sz,
               OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;
    if (!from_sz) {
        ret = OpenSOAPUnicharArrayClear(to_str);
    }
    else {
        const UINT encodingCP = 932;
        int wc_len
            = MultiByteToWideChar(encodingCP,
                                  MB_ERR_INVALID_CHARS,
                                  from_str,
                                  from_sz,
                                  NULL,
                                  0);
        if (!wc_len) {
            DWORD last_error = GetLastError();
            ret = (last_error == ERROR_INSUFFICIENT_BUFFER)
                ? OPENSOAP_MEM_OUTOFRANGE
                : ((last_error == ERROR_NO_UNICODE_TRANSLATION)
                   ? OPENSOAP_INVALID_MB_SEQUENCE
                   : OPENSOAP_PARAMETER_BADVALUE);
        }
        else {
            unichar_t *wc_str = XwcsAlloc(wc_len + 1);
            if (!wc_str) {
                ret = OPENSOAP_MEM_BADALLOC;
            }
            else {
                wc_len
                    = MultiByteToWideChar(encodingCP,
                                          MB_ERR_INVALID_CHARS,
                                          from_str,
                                          from_sz,
                                          wc_str,
                                          wc_len);
                if (!wc_len) {
                    DWORD last_error = GetLastError();
                    ret = (last_error == ERROR_INSUFFICIENT_BUFFER)
                        ? OPENSOAP_MEM_OUTOFRANGE
                        : ((last_error == ERROR_NO_UNICODE_TRANSLATION)
                           ? OPENSOAP_INVALID_MB_SEQUENCE
                           : OPENSOAP_PARAMETER_BADVALUE);
                }
                else {
                    ret = OpenSOAPUnicharArraySetData(to_str,
                                                      wc_str,
                                                      wc_len);
                }

                XwcsFree(&wc_str);
            }
        }
    }
    
    return ret;
}
# else /* !NONGCC_WIN32 */

static
int
UnicharToCP932(const unichar_t * /* [in] */ from_str,
               size_t /* [in] */ from_len,
               OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = UnicharToUSASCII(from_str,
                               from_len,
                               to_str);
        if (OPENSOAP_FAILED(ret)) {
            ret = OPENSOAP_ICONV_NOT_IMPL;
        }
    }

    return ret;
}

static
int
CP932ToUnichar(const unsigned char * /* [in] */ from_str,
               size_t /* [in] */ from_sz,
               OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPUnicharArrayClear(to_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = USASCIIToUnichar(from_str,
                               from_sz,
                               to_str);
        if (OPENSOAP_FAILED(ret)) {
            ret = OPENSOAP_ICONV_NOT_IMPL;
        }
    }

    return ret;
}

# endif /* !NONGCC_WIN32 */

#define IS_SJIS_FIRST(c) \
(0x81 <= (c) && (c) < 0xA0 || 0xE0 <= (c) && (c) < 0xFD)

#define IS_SJIS_SECOND(c) (0x40 <= (c) && (c) < 0xFD && (c) != 0x7F)

#define IS_SJIS_JISX0201KANA(c) (0xA0 <= (c) && (c) < 0xE0)

/*
=begin
= EUC-JP
=end
*/

/*
=begin
--- function#CP932ToEUCJP( from_str, from_sz, to_str)
    CP932(Shift_JIS) -> EUC-JP

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPUnicharArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
CP932ToEUCJP(const unsigned char * /* [in] */ from_str,
             size_t /* [in] */ from_sz,
             OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        const unsigned char *c_end = from_str + from_sz;
        const unsigned char *c_i = from_str;
        while (c_i != c_end && OPENSOAP_SUCCEEDED(ret)) {
#define TMP_BUF_SZ 2
            unsigned char tmp_buf[TMP_BUF_SZ];
            unsigned char *tmp_i = tmp_buf;
            unsigned char c = *c_i++;
            if (IS_SJIS_FIRST(c)) {
                if (c_i == c_end) {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
                else if (!IS_SJIS_SECOND(*c_i)) {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
                else {
                    unsigned char s_c = *c_i++;
                    if (c < 0xF0) {
                        if (c < 0xA0) {
                            c -= '\x70';
                        }
                        else { /* c < '\xF0' */
                            c -= '\xB0';
                        }
                        if (s_c >= 0x80) {
                            --s_c;
                        }
                        c <<= 1;
                        if (s_c >= 0x9E) {
                            s_c -= '\x5E';
                        }
                        else {
                            --c;
                        }
                        s_c -= '\x1F';
                    }
                    *tmp_i++ = c + '\x80';
                    *tmp_i++ = s_c + '\x80';
                }
            }
            else {
                if (IS_SJIS_JISX0201KANA(c)) {
                    *tmp_i++ = '\x8E';
                    *tmp_i++ = c;
                }
                else if (c < 0x80) {
                    *tmp_i++ = c;
                }
                else {
                    ret = OPENSOAP_INVALID_MB_SEQUENCE;
                }
            }
            if (tmp_i != tmp_buf) {
                ret = OpenSOAPByteArrayAppend(to_str,
                                              tmp_buf,
                                              tmp_i - tmp_buf);
            }
#undef  TMP_BUF_SZ
        }
    }

    return ret;
}

/*
=begin
--- function#UnicharToEUCJP( from_str, from_len, to_str)

    :Parameters
        :const unichar_t * [in] ((|from_str|))
        :size_t [in] ((|from_len|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToEUCJP(const unichar_t * /* [in] */ from_str,
               size_t /* [in] */ from_len,
               OpenSOAPByteArrayPtr /* [out] */ to_str) {
    OpenSOAPByteArrayPtr mid_str = NULL;
    int ret = OpenSOAPByteArrayCreate(&mid_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = UnicharToCP932(from_str, from_len, mid_str);
        if (OPENSOAP_SUCCEEDED(ret)) {
            /* CP932 -> EUC-JP */
            const unsigned char *mid_begin = NULL;
            size_t mid_sz = 0;
            ret = OpenSOAPByteArrayGetBeginSizeConst(mid_str,
                                                     &mid_begin,
                                                     &mid_sz);
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = CP932ToEUCJP(mid_begin, mid_sz, to_str);
            }
        }

        OpenSOAPByteArrayRelease(mid_str);
    }

    return ret;
}


/*
=begin
--- function#UnicharToEUCJP( from_str, from_sz, to_str)
    EUC-JP -> CP932(Shift_JIS)

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
#define IS_EUCJP_JISX0212(c) ((c) == 0x8F)
#define IS_EUCJP_JISX0201KANA(c) ((c) == 0x8E)

static
int
EUCJPToCP932(const unsigned char * /* [in] */ from_str,
             size_t /* [in] */ from_sz,
             OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        const unsigned char *c_end = from_str + from_sz;
        const unsigned char *c_i = from_str;
        while (c_i != c_end && OPENSOAP_SUCCEEDED(ret)) {
#define TMP_BUF_SZ 2
            unsigned char tmp_buf[TMP_BUF_SZ];
            unsigned char *tmp_i = tmp_buf;
            unsigned char c = *c_i++;
            if (IS_EUCJP_JISX0212(c)) {
                ret = OPENSOAP_INVALID_MB_SEQUENCE;
            }
            else if (IS_EUCJP_JISX0201KANA(c)) {
                if (c_i == c_end) {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
                else {
                    *tmp_i++ = *c_i++;
                }
            }
            else if (c < 0x80) { /* ASCII */
                *tmp_i++ = c;
            }
            else {
                if (c_i == c_end) {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
                else {
                    unsigned char p = *c_i++;
                    c &= 0x7f;
                    p &= 0x7f;
                    
                    c -= ' '; /* KU */
                    p += 0x1F; /* 63(0x3F) - ' '(0x20) */

                    if (c & 1) { /* KU odd */
                        ++c;
                        if (p >= 0x3F + 0x40) { /* ten >= 64 */
                            ++p;
                        }
                    }
                    else { /* KU even */
                        p += 0x5F; /* 158 - 63 */
                    }
                    c >>= 1;
                    c += 0x80L;
                    if (c > 0x9F) {
                        c += 0x40;
                    }

                    *tmp_i++ = c;
                    *tmp_i++ = p;
                }
            }
            if (tmp_i != tmp_buf) {
                ret = OpenSOAPByteArrayAppend(to_str,
                                              tmp_buf,
                                              tmp_i - tmp_buf);
            }
#undef  TMP_BUF_SZ
        }
    }

    return ret;
}

/*
=begin
--- function#EUCJPToUnichar( from_str, from_sz, to_str)
    EUC-JP -> Unichar

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPUnicharArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
EUCJPToUnichar(const unsigned char * /* [in] */ from_str,
               size_t /* [in] */ from_sz,
               OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    OpenSOAPByteArrayPtr mid_str = NULL;
    int ret = OpenSOAPByteArrayCreate(&mid_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = EUCJPToCP932(from_str, from_sz, mid_str);
        if (OPENSOAP_SUCCEEDED(ret)) {
            const unsigned char *mid_beg = NULL;
            size_t mid_sz = 0;
            ret = OpenSOAPByteArrayGetBeginSizeConst(mid_str,
                                                     &mid_beg,
                                                     &mid_sz);
            
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = CP932ToUnichar(mid_beg, mid_sz, to_str);
            }
        }

        OpenSOAPByteArrayRelease(mid_str);
    }

    return ret;
}

/*
=begin
= Unicode
=end
*/
/*
=begin
--- function#UCS2toUCS4( first, last, ucs4)
    UCS2    <-> UCS4
    unichar_t <-> unsigned long

    :Parameters
        :const unichar_t **[in,out] ((|first|))
        :const unichar_t * [in] ((|last|))
        :unsigned long * [out] ((|ucs4|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UCS2toUCS4(const unichar_t ** /* [in, out] */ first,
           const unichar_t * /* [in] */ last,
           unsigned long * /* [out] */ ucs4) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (first && *first && last && *first < last && ucs4) {
        const unichar_t *i = *first;
        unichar_t hc = *i++;
        ret = OPENSOAP_NO_ERROR;
        if (0xd800 <= hc && hc < 0xdc00) { /* High Surrogates */
            if (i < last) {
                unichar_t lc = *i++;
                if (0xdc00 <= lc && lc < 0xe000) { /* Low Surrogates */
                    *ucs4 = (hc - 0xd800) * 0x400 + (lc - 0xdc00) + 0x10000L;
                }
                else {
                    /* High Surrogates without Low Surrogates */
                    ret = OPENSOAP_INVALID_MB_SEQUENCE;
                }
            }
            else { /* not Low Surrogates */
                ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
            }
        }
        else if (0xdc00 <= hc && hc < 0xe000) { /* Low Surrogates without
                                                 * High Surrogates */
            ret = OPENSOAP_INVALID_MB_SEQUENCE;
        }
        else {                          /* not Surrogates */
            *ucs4 = hc;
        }
        if (OPENSOAP_SUCCEEDED(ret)) {
            *first = i;
        }
    }

    return ret;
}

/*
=begin
--- function#UCS4toUCS2( ucs4, first, last)
    UCS2    <-> UCS4
    unichar_t <-> unsigned long

    :Parameters
        :unsigned long * [in] ((|ucs4|))
        :const unichar_t **[in,out] ((|first|))
        :const unichar_t * [in] ((|last|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UCS4toUCS2(unsigned long /* [in] */ ucs4,
           unichar_t ** /* [in, out] */ first,
           unichar_t * /* [in] */ last) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (first && *first && last && *first < last) {
        unichar_t *i = *first;
        ret = OPENSOAP_NO_ERROR;
        if (ucs4 < 0x10000L) {
            *i++ = (unichar_t)ucs4;
        }
        else if (i + 1 < last) {
            const unsigned long div_num = 0x400L;
#if 1
            unsigned long d_quot = ucs4 / div_num;
            unsigned long d_rem  = ucs4 % div_num;
#else
            ldiv_t d = ldiv(ucs4, div_num);
            unsigned long d_quot = d.quot;
            unsigned long d_rem  = d.rem;
#endif
            *i++ = (unichar_t)((d_quot - 0x40L) + 0xd800);
            *i++ = (unichar_t)(d_rem + 0xdc00);
        }
        else {
            ret = OPENSOAP_MEM_OUTOFRANGE;
        }
        if (OPENSOAP_SUCCEEDED(ret)) {
            *first = i;
        }
    }
    
    return ret;
}

/*
=begin
= UTF-8
=end
*/
/*
=begin
--- function#UnicharToUTF8( from_str, from_len, to_str)

	Unichar -> UTF-8

    :Parameters
        :const unichar_t *  [in]  ((|from_str|))
        :size_t  [in] ((|from_len|))
        :OpenSOAPByteArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/

#define UTF8_FOLLOW_SIZE(c) \
((c > 0xffffL) ? 3 \
: ((c > 0x7ff) ? 2 \
: ((c > 0x7f) ? 1 : 0)))

static
int
UnicharToUTF8(const unichar_t * /* [in] */ from_str,
              size_t /* [in] */ from_len,
              OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
#define TMP_BUF_SZ 4
        unsigned char tmp_buf[TMP_BUF_SZ];
        const unichar_t *c_i = from_str;
        const unichar_t *ed = from_str + from_len;

        while (c_i != ed && OPENSOAP_SUCCEEDED(ret)) {
            unsigned long c = 0;
            ret = UCS2toUCS4(&c_i, ed, &c);
            if (OPENSOAP_SUCCEEDED(ret)) {
                static const long FIRST_DIV_NUM[] = {
                    1L,
                    0x40L,
                    0x1000L,
                    0x4000L
                };
                static const unsigned char FIRST_MASK[] = {
                    '\0',
                    '\xc0',
                    '\xe0',
                    '\xf0'
                };
                int append_size = 0;
                int add_num = UTF8_FOLLOW_SIZE(c);
                long div_num = FIRST_DIV_NUM[add_num];
                unsigned char mask_val = FIRST_MASK[add_num];

                while (1) {
#if 1
                    unsigned long d_quot = c / div_num;
                    unsigned long d_rem  = c % div_num;
#else
                    ldiv_t d = ldiv(c, div_num);
                    unsigned long d_quot = d.quot;
                    unsigned long d_rem  = d.rem;
#endif                    
                    tmp_buf[append_size++]
                        = (unsigned char)d_quot | mask_val;
                    if (!add_num) {
                        break;
                    }
                    c = d_rem;
                    --add_num;
                    div_num = (1 << 6 * add_num);
                    mask_val = '\x80';
                }
                ret = OpenSOAPByteArrayAppend(to_str,
                                              tmp_buf,
                                              append_size);
            }
        }
#undef TMP_BUF_SZ
    }
    
    return ret;
}

#define IS_UTF8_FOLLOW(c) (((c) & 0xc0) == 0x80)
#define IS_UTF8_ASCII(c) (!((c) & 0x80))
#define UTF8_SIZE(c) \
((!((c) & 0x80)) ? 1 \
: ((((c) & 0xe0) == 0xc0) ? 2 \
: ((((c) & 0xf0) == 0xe0) ? 3 \
: ((((c) & 0xf8) == 0xf0) ? 4 : 0))))

#define UNICODE_BOM (0xfeff)
#define UTF16_BOM_LE (0xfffe)
#define UTF32_BOM_LE (0xfffe0000L)

/*
=begin
--- function#UTF8ToUnichar( from_str, from_sz, to_str)
	Unichar <- UTF-8

    :Parameters
        :const unsigned char *  [in]  ((|from_str|))
        :size_t  [in]  ((|from_sz|))
        :OpenSOAPUnicharArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UTF8ToUnichar(const unsigned char * /* [in] */ from_str,
              size_t /* [in] */ from_sz,
              OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPUnicharArrayClear(to_str);

    if (OPENSOAP_SUCCEEDED(ret)) {
        ret = OPENSOAP_PARAMETER_BADVALUE;
        if (from_str && from_sz) {
            int is_bom_checked = 0;
            const unsigned char *c_i = from_str;
            const unsigned char *c_end = from_str + from_sz;

            ret = OPENSOAP_NO_ERROR;
            while (c_i != c_end && OPENSOAP_SUCCEEDED(ret)) {
                unsigned char c = *c_i;
                size_t utf8_sz = UTF8_SIZE(c);
                unsigned long ucs4 = 0;
                if (!utf8_sz) {
                    ret = OPENSOAP_INVALID_MB_SEQUENCE;
                }
                else {
                    --utf8_sz;
                    if (c_i + utf8_sz < c_end) {
                        ucs4 = (c
                                & (0xff >> (utf8_sz + (utf8_sz ? 2 : 1))));
                        for (++c_i; utf8_sz && IS_UTF8_FOLLOW(*c_i);
                             --utf8_sz, ++c_i) {
                            ucs4 <<= 6;
                            ucs4 += (0x3f & *c_i);
                        }
                        if (utf8_sz) {
                            ret = OPENSOAP_INVALID_MB_SEQUENCE;
                        }
                    }
                    else {
                        ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                    }
                }
                
                if (OPENSOAP_SUCCEEDED(ret)) {
                    if (is_bom_checked || ucs4 != UNICODE_BOM) {
#define TMP_STRBUF_SZ 2
                        unichar_t strbuf[TMP_STRBUF_SZ + 1];
                        unichar_t * const buf_ed = &strbuf[TMP_STRBUF_SZ];
                        unichar_t *buf_i = strbuf;

                        ret = UCS4toUCS2(ucs4, &buf_i, buf_ed);
                        if (OPENSOAP_SUCCEEDED(ret)) {
                            ret =
                                OpenSOAPUnicharArrayAppend(to_str,
                                                           strbuf,
                                                           (buf_i - strbuf));
                        }
#undef TMP_STRBUF_SZ
                    }
                    is_bom_checked = 1;
                }
            }
        }
    }
    
    return ret;
}

/*
=begin
--- function#UTF8ToUSASCII( from_str, from_sz, to_str)
    to US-ASCII if input not US-ASCII then error

    :Parameters
        :const unsigned char * [in] ((|from_str|))
        :size_t [in] ((|from_sz|))
        :OpenSOAPByteArrayPtr [out] ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UTF8ToUSASCII(const unsigned char * /* [in] */from_str,
              size_t /* [in] */ from_sz,
              OpenSOAPByteArrayPtr /* [out] */ to_str) {
    /* BOM check */
    static const unsigned char UTF8_BOM[] = "\xef\xbb\xbf";
    static size_t UTF8_BOM_SIZE = sizeof(UTF8_BOM) - 1;
    if (from_str && from_sz > UTF8_BOM_SIZE
        && strncmp(from_str, UTF8_BOM, UTF8_BOM_SIZE) == 0) {
        from_str += UTF8_BOM_SIZE;
    }

    return USASCIIToUSASCII(from_str, from_sz, to_str);
}

/*
=begin
= UTF-16
=end
*/
/*
=begin
--- function#UnicharToUTF16( from_str, from_len, to_str)

	Unichar -> UTF-16

    :Parameters
        :const unichar_t *  [in]  ((|from_str|))
        :size_t  [in] ((|from_len|))
        :OpenSOAPByteArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToUTF16(const unichar_t * /* [in] */ from_str,
              size_t /* [in] */ from_len,
              OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        unichar_t bom = UNICODE_BOM;
        ret = OpenSOAPByteArrayAppend(to_str,
                                      (unsigned char *)(&bom),
                                      sizeof(unichar_t));
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret =
                OpenSOAPByteArrayAppend(to_str,
                                        (unsigned char *)from_str,
                                        from_len * sizeof(unichar_t));
        }
    }
    
    return ret;
}

/*
=begin
--- function#UTF16ToUnichar( from_str, from_sz, to_str)
	Unichar <- UTF-16

    :Parameters
        :const unsigned char *  [in]  ((|from_str|))
        :size_t  [in]  ((|from_sz|))
        :OpenSOAPUnicharArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UTF16ToUnichar(const unsigned char * /* [in] */ from_str,
              size_t /* [in] */ from_sz,
              OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPUnicharArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        if (from_str && from_sz) {
            int is_le = 0;                  /* Is UTF-16LE? */
            int chk_bom = 0;
            const unsigned char *i = from_str;
            const unsigned char *ed = from_str + from_sz;

            ret = OPENSOAP_NO_ERROR;
            while (i < ed && OPENSOAP_SUCCEEDED(ret)) {
                unichar_t hc = *i++;
                if (i < ed) {
                    unichar_t lc = *i++;
                    unichar_t c = (is_le) ? ((lc << 8) + hc)
                        :((hc << 8) + lc);
                    if (!chk_bom) {
                        chk_bom = 1;
                        is_le = (c == UTF16_BOM_LE);
                        if (c == UNICODE_BOM || is_le) {
                            continue;
                        }
                    }
                    ret = OpenSOAPUnicharArrayAppend(to_str, &c, 1);
                }
                else {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
            }
        }
        else {
            ret = OPENSOAP_PARAMETER_BADVALUE;
        }
    }
    
    return ret;
}

/*
=begin
= UTF-32
=end
*/
/*
=begin
--- function#UnicharToUTF32( from_str, from_len, to_str)

	Unichar -> UTF-32

    :Parameters
        :const unichar_t *  [in]  ((|from_str|))
        :size_t  [in] ((|from_len|))
        :OpenSOAPByteArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UnicharToUTF32(const unichar_t * /* [in] */ from_str,
               size_t /* [in] */ from_len,
               OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPByteArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        const unichar_t *i  = from_str;
        const unichar_t *ed = from_str + from_len;
        unsigned long ucs4 = UNICODE_BOM;

        ret = OpenSOAPByteArrayAppend(to_str,
                                      (unsigned char *)(&ucs4),
                                      sizeof(ucs4));
        while (i < ed && OPENSOAP_SUCCEEDED(ret)) {
            if (OPENSOAP_FAILED(ret)) {
                break;
            }
            ret = UCS2toUCS4(&i, ed, &ucs4);
            if (OPENSOAP_FAILED(ret)) {
                break;
            }
            ret = OpenSOAPByteArrayAppend(to_str,
                                          (unsigned char *)(&ucs4),
                                          sizeof(ucs4));
        }
    }
    
    return ret;
}

/*
=begin
--- function#UTF32ToUnichar( from_str, from_sz, to_str)
	Unichar <- UTF-32

    :Parameters
        :const unsigned char *  [in]  ((|from_str|))
        :size_t  [in]  ((|from_sz|))
        :OpenSOAPUnicharArrayPtr  [out]  ((|to_str|))
    :Return Value
      :int
	    Return Code Is ErrorCode
	:Note
	  This Function Is Static Function
=end
*/
static
int
UTF32ToUnichar(const unsigned char * /* [in] */ from_str,
               size_t /* [in] */ from_sz,
               OpenSOAPUnicharArrayPtr /* [out] */ to_str) {
    int ret = OpenSOAPUnicharArrayClear(to_str);
    if (OPENSOAP_SUCCEEDED(ret)) {
        if (from_str && from_sz) {
            int is_le = 0;                  /* Is UTF-32LE? */
            int chk_bom = 0;
            const unsigned char *i = from_str;
            const unsigned char *ed = from_str + from_sz;

            ret = OPENSOAP_NO_ERROR;
            while (i < ed && OPENSOAP_SUCCEEDED(ret)) {
                const unsigned char *i_ed = i + 4;
                if (i_ed < ed + 1) {
                    unsigned long c = 0;
#define TMP_STRBUF_SZ 2
                    unichar_t strbuf[TMP_STRBUF_SZ + 1];
                    unichar_t * const buf_ed = &strbuf[TMP_STRBUF_SZ];
                    unichar_t *buf_i = strbuf;
                    
                    if (is_le) {        /* LE convert */
                        for (; i != i_ed; ++i) {
                            c <<= 8;
                            c += *i;
                        }
                    }
                    else {              /* BE convert */
                        unsigned long t = 0x1000000L;
                        for (; i != i_ed; ++i) {
                            c += t * *i;
                            t >>= 8;
                        }
                    }
                    if (!chk_bom) {
                        chk_bom = 1;
                        is_le = (c == UTF32_BOM_LE);
                        if (c == UNICODE_BOM || is_le) {
                            continue;
                        }
                    }

                    ret = UCS4toUCS2(c, &buf_i, buf_ed);
                    if (OPENSOAP_SUCCEEDED(ret)) {
                        ret = OpenSOAPUnicharArrayAppend(to_str,
                                                         strbuf,
                                                         (buf_i - strbuf));
                    }
#undef TMP_STRBUF_SZ
                }
                else {
                    ret = OPENSOAP_INCOMPLETE_MB_SEQUENCE;
                }
            }
        }
        else {
            ret = OPENSOAP_PARAMETER_BADVALUE;
        }
    }
    
    return ret;
}

/*---------------------------------------------------*/


typedef struct {
    const char *type_name;
    FromUnicharFunc fromUnichar;
    ToUnicharFunc   toUnichar;
} ConverterMapItem;

typedef struct {
    const char *from_enc;
    const char *to_enc;
    MBConvFunc  mb_conv;
} MBConverterMapItem;

static
const ConverterMapItem
ConverterMap[] = {
    { "US-ASCII", UnicharToUSASCII, USASCIIToUnichar},
    { "ISO-8859-1", UnicharToLatin1, Latin1ToUnichar},
    { "Shift_JIS", UnicharToCP932, CP932ToUnichar},
    { "EUC-JP", UnicharToEUCJP, EUCJPToUnichar},
    { "UTF-8", UnicharToUTF8, UTF8ToUnichar},
    { "UTF-16", UnicharToUTF16, UTF16ToUnichar},
    { "UTF-32", UnicharToUTF32, UTF32ToUnichar},
    { NULL, NULL, NULL}
};

static
MBConverterMapItem
MBConverterMap[] = {
    {"US-ASCII", "Shift_JIS", USASCIIToUSASCII},
    {"US-ASCII", "EUC-JP", USASCIIToUSASCII},
    {"US-ASCII", "UTF-8", USASCIIToUSASCII},
    {"Shift_JIS", "US-ASCII", USASCIIToUSASCII},
    {"Shift_JIS", "EUC-JP", CP932ToEUCJP},
    {"EUC-JP", "US-ASCII", USASCIIToUSASCII},
    {"EUC-JP", "Shift_JIS", EUCJPToCP932},
    {"UTF-8", "US-ASCII", UTF8ToUSASCII},
    {NULL, NULL, NULL}
};

/*
=begin
--- function#GetConverter( enc_type)

    :Parameters
        :const char *  [in]  ((|enc_type|))
    :Return Value
      :const ConverterMapItem *
	:Note
	  This Function Is Static Function
=end
*/
static
const ConverterMapItem *
GetConverter(const char * /* [in] */ enc_type) {
    const ConverterMapItem *ret = ConverterMap;

    for (; ret->type_name
             && (strcasecmp(ret->type_name, enc_type) != 0); ++ret) {
    }

    if (!(ret->type_name)) {
        ret = NULL;
    }

    return ret;
}

/*
=begin
--- function#GetMBConverter( from_enc, to_enc)

    :Parameters
        :const char * [in]  ((|from_enc|))
        :const char * [in]  ((|to_enc|))
    :Return Value
      :MBConvFunc
	:Note
	  This Function Is Static Function
=end
*/
static
MBConvFunc
GetMBConverter(const char *from_enc,
               const char *to_enc) {
    MBConvFunc  ret = NULL;
    MBConverterMapItem *i = MBConverterMap;
    for (; i->from_enc && i->to_enc; ++i) {
        if (strcasecmp(i->from_enc, from_enc) == 0
            && strcasecmp(i->to_enc, to_enc) == 0) {
            ret = i->mb_conv;
            break;
        }
    }

    return ret;
}

/*-------------------------------------------------------------------------*/

/*
--- function#OpenSOAPStringConvertCharEncodingWithSize(from_enc, from_str, to_enc, to_str)
    static function.
    Convert character encoding.
    This function isn't include OpenSOAPString member,
    but this has something to do with String processing.

    :Parameters
      :const char * [in] ((|from_enc|))
        Convert source character encoding. If NULL or "" then current
        locale encoding.
      :const unsigned char * [in] ((|from_beg|))
        Convert source string data.
      :const unsigned char * [in] ((|from_sz|))
        Convert source string data size.
      :const char * [in] ((|to_enc|))
        Convert distination character encoding. If NULL or "" then current
        locale encoding.
      :OpenSOAPByteArrayPtr [out] ((|to_str|))
        Convert distination string data. If from_enc equal to to_enc,
        then copy from_str to to_str.
    :Return Value
      :^
            int
      error code.

=end
 */
static
int
OpenSOAPStringConvertCharEncodingWithSize(const char * /* [in] */ from_enc,
                                          const unsigned char *
                                          /* [in] */ from_beg,
                                          size_t /* [in] */ from_sz,
                                          const char * /* [in] */ to_enc,
                                          OpenSOAPByteArrayPtr
                                          /* [out] */ to_str) {
    int ret = OPENSOAP_YET_IMPLEMENTATION;
    if (!from_sz) {
        ret = OpenSOAPByteArrayClear(to_str);
    }
    else {
        const char *def_enc = GetDefaultCodeSet();

        if (!from_enc) {
            from_enc = def_enc;
        }
        if (!to_enc) {
            to_enc = def_enc;
        }
        if (strcmp(from_enc, to_enc) == 0) {
            /* duplicate */
            ret = OpenSOAPByteArrayClear(to_str);
            if (OPENSOAP_SUCCEEDED(ret)) {
                ret = OpenSOAPByteArrayAppend(to_str,
                                              from_beg,
                                              from_sz);
            }
        }
        else {
            MBConvFunc mb_conv = GetMBConverter(from_enc, to_enc);
            if (mb_conv) {
                ret = (mb_conv)(from_beg, from_sz, to_str);
            }
            else {
                const ConverterMapItem *from_conv
                    = GetConverter(from_enc);
                const ConverterMapItem *to_conv
                    = GetConverter(to_enc);
                if (!from_conv || !to_conv) {
                    ret = OPENSOAP_UNKNOWN_CHARENCODE;
                }
                else {
                    OpenSOAPUnicharArrayPtr mid_str = NULL;
                    ret = OpenSOAPUnicharArrayCreate(&mid_str);
                    if (OPENSOAP_FAILED(ret)) {
                        ret = OPENSOAP_MEM_BADALLOC;
                    }
                    else {
                        ret = (from_conv->toUnichar)(from_beg, from_sz, mid_str);
                        if (OPENSOAP_SUCCEEDED(ret)) {
                            const unichar_t *mid_beg = NULL;
                            size_t mid_sz = 0;
                            ret = OpenSOAPUnicharArrayGetBeginSizeConst(mid_str,
                                                                        &mid_beg,
                                                                        &mid_sz);
                            if (OPENSOAP_SUCCEEDED(ret)) {
								/* clear to_str */
								ret = OpenSOAPByteArrayClear(to_str);
								if (OPENSOAP_SUCCEEDED(ret)) {
									ret = (to_conv->fromUnichar)(mid_beg,
																 mid_sz,
																 to_str);
								}
                            }
                        }

                        OpenSOAPUnicharArrayRelease(mid_str);
                    }
                }
            }
        }
    }
    
    return ret;
}

#endif /* !HAVE_ICONV_H */

/*
=begin
--- function#OpenSOAPStringConvertCharEncoding(from_enc, from_str, to_enc, to_str)
    Convert character encoding.
    This function isn't include OpenSOAPString member,
    but this has something to do with String processing.

    :Parameters
      :const char * [in] ((|from_enc|))
        Convert source character encoding. If NULL or "" then current
        locale encoding.
      :OpenSOAPByteArrayPtr [in] ((|from_str|))
        Convert source string data.
      :const char * [in] ((|to_enc|))
        Convert distination character encoding. If NULL or "" then current
        locale encoding.
      :OpenSOAPByteArrayPtr [out] ((|to_str|))
        Convert distination string data. If from_enc equal to to_enc,
        then copy from_str to to_str.
    :Return Value
      :int
	    Return Code Is ErrorCode
=end
 */
extern
int
OPENSOAP_API
OpenSOAPStringConvertCharEncoding(const char * /* [in] */ from_enc,
                                  OpenSOAPByteArrayPtr /* [in] */ from_str,
                                  const char * /* [in] */ to_enc,
                                  OpenSOAPByteArrayPtr /* [out] */ to_str) {
    int ret = OPENSOAP_PARAMETER_BADVALUE;

    if (from_str && to_str) {
        const unsigned char *from_beg = NULL;
        size_t  from_sz = 0;
        
        ret = OpenSOAPByteArrayGetBeginSizeConst(from_str,
                                                 &from_beg, &from_sz);
        if (OPENSOAP_SUCCEEDED(ret)) {
            ret = OpenSOAPStringConvertCharEncodingWithSize(from_enc,
                                                            from_beg,
                                                            from_sz,
                                                            to_enc,
                                                            to_str);
        }
    }

    return ret;
}
