/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: ChannelSelector.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#if defined(WIN32)
#include <windows.h>
#else
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#endif

#include <iostream>
#include <sys/types.h>
#include <fcntl.h>

#include "MapUtil.h"
#include "ServerCommon.h"
#include "ChannelCommon.h"
#include "ChannelSelector.h"

//temp.
#if defined(HAVE_FIFO_IMPL)
#include "FIFODescriptor.h"
#include "FIFOSelector.h"
#endif


using namespace OpenSOAP;
using namespace std;

//for LOG
static const std::string CLASS_SIG = "ChannelSelector";


ChannelSelector::ChannelSelector()
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::" << CLASS_SIG << endl;
#endif /* DEBUG */  

}

ChannelSelector::ChannelSelector(
#if defined(WIN32)
				 const int addr)
#else
                                 const std::string& addr)
#endif
  : socketAddr_(addr)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::" << CLASS_SIG << endl;
#endif /* DEBUG */  



}

ChannelSelector::~ChannelSelector()
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::~" << CLASS_SIG << endl;
#endif /* DEBUG */  

}

void 
ChannelSelector::setSocketAddr(
#if defined(WIN32)
			       const int addr)
#else
                               const std::string& addr)
#endif
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::setSocketAddr() arg=[" 
       << addr << "]" << endl;
#endif /* DEBUG */

  socketAddr_ = addr;
}

int 
ChannelSelector::connectChannelManager()
{
#if defined(WIN32)
  typedef SOCKADDR SockAddr;
  typedef SOCKADDR_IN SockAddrAf;

	SockAddrAf servAddr;

	SOCKET sockfd = INVALID_SOCKET;
	WSADATA data;
	if (WSAStartup(0x101,&data)) {
		cerr << "** E **: WSAStartup failed." << endl;
		throw exception();
	}
	int af = AF_INET;
	int addrLen = sizeof(SockAddrAf);
	int servAddrLen = sizeof(servAddr);
	char hostname[256];
	if (gethostname(hostname, sizeof(hostname))) {
	    cerr << TAG_ERR << CLASS_SIG 
		 << "::connectChannelManager gethostname failed" << endl;
	    return -1;
	}
	servAddr.sin_family = af;
	PHOSTENT phe = gethostbyname(hostname);
	memcpy(&(servAddr.sin_addr), phe->h_addr, phe->h_length);
	servAddr.sin_port = htons(socketAddr_);
#else
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_un SockAddrAf;

  int sockfd = int();
  SockAddrAf servAddr;
  int af = AF_UNIX;

  memset((char*)&servAddr, 0, sizeof(servAddr));
  servAddr.sun_family = af;
  strcpy(servAddr.sun_path, socketAddr_.c_str());

#if HAVE_TYPE_OF_ADDRLEN_AS_SOCKLEN_T
  socklen_t servAddrLen = socklen_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_SIZE_T
  size_t servAddrLen = size_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_INT
  int servAddrLen = int();
#endif
  
  servAddrLen = sizeof(servAddr.sun_family) + strlen(servAddr.sun_path);
#endif

  if (0 > (sockfd = socket(af, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectChannelManager open socket failed" << endl;
    throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::connectChannelManager: socket open OK!" << endl;
#endif
  
  if (0 > connect(sockfd, (SockAddr*)&servAddr, servAddrLen)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::connectChannelManager connect failed" << endl;
    throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::connectChannelManager: connect OK!" << endl;
#endif

    return sockfd;
}

std::string ChannelSelector::makeSendOpenMessage(const int waitSecond)
{
  return "";
}

std::string ChannelSelector::makeSendCloseMessage()
{
  return "";
}

int ChannelSelector::open(ChannelDescriptor& chnlDesc, const int waitSecond)
{


  // timeout l̃`FbN
  if (ChannelDescriptor::NO_WAIT != waitSecond &&
      ChannelDescriptor::WAIT_NOLIMIT != waitSecond &&
      0 > waitSecond) {

    cerr << TAG_ERR << CLASS_SIG << "::open: invalid timeout second" << endl;
    return -1;
  }



  int sockfd = connectChannelManager();

  if (0 > sockfd) {

    return sockfd;
  }

  std::string sendMessage = makeSendOpenMessage(waitSecond);


#ifdef DEBUG
  cerr << CLASS_SIG << "::open: sendMessage=["
       << sendMessage << "]" << endl;
#endif /* DEBUG */  

  if (0 > write(sockfd, sendMessage)) {
    cerr << TAG_ERR << CLASS_SIG << "::open: write failed" << endl;
    return -1;
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::open write done." << endl;
#endif

  std::string recvMessage;
  int recvLen = read(sockfd, recvMessage);

#ifdef DEBUG
  cerr << CLASS_SIG << "::open: read=[" << recvMessage 
       << "] len=(" << recvLen << ")" << endl;
#endif

  MapUtil mapUtil(OpenSOAP::ChnlCommon::ITEM_DELMIT, 
		  OpenSOAP::ChnlCommon::KEY_DELMIT);
  mapUtil.insertItem(recvMessage);
  //ʏ
  if (!mapUtil.exists(OpenSOAP::ChnlCommon::RESULT)) {
    cerr << TAG_ERR << CLASS_SIG << "::open invalid read message. "
	 << "RESUT not found. msg=[" << recvMessage << "]" << endl;
    return -1;
  }
  //`l^Cv
  if (!mapUtil.exists(OpenSOAP::ChnlCommon::CHNL_TYPE)) {
    cerr << TAG_ERR << CLASS_SIG << "::open invalid read message. "
	 << "CHNL_TYPE not found. msg=[" << recvMessage << "]" << endl;
    return -1;
  }

  std::string result = mapUtil[OpenSOAP::ChnlCommon::RESULT];
  std::string type = mapUtil[OpenSOAP::ChnlCommon::CHNL_TYPE];
  if (OpenSOAP::ChnlCommon::SUCCESS == result) {

    if (OpenSOAP::ChnlCommon::UNIX_SOCKET == type) {
      chnlDesc.setReadFd(sockfd);
      chnlDesc.setWriteFd(sockfd);
    }
    else if (OpenSOAP::ChnlCommon::FIFO == type) {
      //
      //̕͏Kv
      //
#if defined(HAVE_FIFO_IMPL)
      FIFODescriptor fifoDesc; 
      FIFOSelector* fifoSelector = new FIFODownsideSelector(6500);
      if (0 != fifoSelector->open(fifoDesc, waitSecond)) {
	cerr << TAG_ERR << CLASS_SIG << "::open fifo open failed" << endl;
	return -1;
      }

      chnlDesc.setReadFd(fifoDesc.getReadFIFO());
      chnlDesc.setWriteFd(fifoDesc.getWriteFIFO());
      chnlDesc.setId(fifoDesc.getId());

      delete fifoSelector;
#endif
    }
    else if (OpenSOAP::ChnlCommon::INET_SOCKET == type) {
      // not support
      cerr << TAG_WRN << CLASS_SIG << "::open invalid read message. "
	   << "not supported CHNL_TYPE . CHNL_TYPE=[" << type << "]" << endl;
      return -1;
    }
    else {
      cerr << TAG_ERR << CLASS_SIG << "::open invalid read message. "
	   << "ilegal CHNL_TYPE . CHNL_TYPE=[" << type << "]" << endl;
      return -1;
    }
  } 
  else if (OpenSOAP::ChnlCommon::FAILURE == result) {
    return 1;
  }
  else {
    cerr << TAG_ERR << CLASS_SIG << "::open invalid result. RESULT=["
	 << result << "]" << endl;
    return -1;
  }

  return 0;
}

int ChannelSelector::close(ChannelDescriptor& chnlDesc)
{
  //socketclose
  //std::close(chnlDesc.getWriteFd());
  //std::close(chnlDesc.getReadFd());

  int sockfd = connectChannelManager();
  if (0 > sockfd) {
    return sockfd;
  }


  std::string sendMessage = makeSendCloseMessage();



  if (0 > write(sockfd, sendMessage)) {

    cerr << TAG_ERR << CLASS_SIG << "::close: write failed" << endl;
    return -1;
  }



  std::string recvMessage;
  int recvLen = read(sockfd, recvMessage);



  MapUtil mapUtil(OpenSOAP::ChnlCommon::ITEM_DELMIT,
		  OpenSOAP::ChnlCommon::KEY_DELMIT);
  mapUtil.insertItem(recvMessage);
  if (!mapUtil.exists(OpenSOAP::ChnlCommon::RESULT)) {

    cerr << TAG_ERR << CLASS_SIG << "::close invalid read message. msg=["
	 << recvMessage << "]" << endl;
    return -1;
  }

  std::string result = mapUtil[OpenSOAP::ChnlCommon::RESULT];

  if (OpenSOAP::ChnlCommon::SUCCESS == result) {

    if (mapUtil.exists(OpenSOAP::ChnlCommon::CHNL_TYPE) &&
	OpenSOAP::ChnlCommon::FIFO == mapUtil[OpenSOAP::ChnlCommon::CHNL_TYPE]) {
      //
      //̕͏Kv
      //
#if defined(HAVE_FIFO_IMPL)
      FIFODescriptor fifoDesc; 
      fifoDesc.setReadFIFO(chnlDesc.getReadFd());
      fifoDesc.setWriteFIFO(chnlDesc.getWriteFd());
      fifoDesc.setId(chnlDesc.getId());

      FIFOSelector* fifoSelector = new FIFODownsideSelector(6500);
      fifoSelector->close(fifoDesc);
      delete fifoSelector;
#endif
    }
    else {

      //socketclose
#if defined(WIN32)
		closesocket(chnlDesc.getWriteFd());
#else
      std::close(chnlDesc.getWriteFd());
      //std::close(chnlDesc.getReadFd());
#endif
    }
  } 
  else if (OpenSOAP::ChnlCommon::FAILURE == result) {

    return 1;
  }
  else {

    cerr << TAG_ERR << CLASS_SIG << "::open invalid result. RESULT=["
	 << result << "]" << endl;
    return -1;
  }


  return 0;
}


// End of ChannelSelector.cpp
