/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SOAPMessage.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#ifndef SOAP_MESSAGE_H
#define SOAP_MESSAGE_H

#include <string>
#include <vector>

#include "Defines.h"

OPENSOAP_VAR
extern
std::string
createSOAPFaultMessage (std::string faultcodeStr, std::string faultstringStr,
			std::string faultactorStr, std::string detailStr);

OPENSOAP_VAR
extern
#if 1
bool
extractOperationAndParam (std::string soapMsg,
			  std::vector<std::string>& operationNames,
			  std::vector<std::string>& paramNames,
			  std::vector<std::string>& paramValues,
			  std::string* nsPtr = NULL);
#else
bool
extractOperationAndParam (std::string soapMsg,
			  std::vector<std::string>& operationNames,
			  std::vector<std::string>& paramNames,
			  std::vector<std::string>& paramValues);
#endif
/*
  check Messaage ID
 */
OPENSOAP_VAR
extern
bool
isItMsgID(const std::string soapMsg, bool& itIsMsgID);

/*
  check sync or async, and ttl info
 */
OPENSOAP_VAR
extern
bool
getAsyncInfoFromHeader(const std::string soapMsg, 
		       bool& asyncHeaderExist, bool& isAsync);

/*
  check forward
*/
OPENSOAP_VAR
extern
bool
getFwdInfoFromHeader(const std::string soapMsg,
		     bool& fwdPathExist, std::vector<std::string>& fwdPath,
		     bool& hopCountExist, unsigned int& hopCount);
/*
  Received Path Ɋւ𒲂ׂ
*/
OPENSOAP_VAR
bool
getReceivedPathInfo(const std::string soapMsg, 
		    bool& receivedPathExist, 
		    std::vector<std::string>& receivedPath);

/*
  check TTL
 */
OPENSOAP_VAR
extern
bool
getTTLInfo(const std::string soapMsg,
	   bool& ttlHeaderExist, unsigned int& ttl, std::string& ttlType);

// get message ID from SOAPmessage
OPENSOAP_VAR
extern
std::string
getMsgIDContent(const std::string soapMsg);

//create fault message for timeout
OPENSOAP_VAR
extern
std::string
makeTimeoutMessage();

//create fault message for operation not exist
OPENSOAP_VAR
extern
std::string
makeOperationNotFoundMessage();

//create fault message for invalid request message
OPENSOAP_VAR
extern
std::string
makeInvalidRequestMessage();

//----- create Falut Messgage of No Entry -----//
OPENSOAP_VAR
extern
std::string
makeNoEntryMessage();

//----- create Falut Messgage in the case of -----//
//----- Response from Serive is Empty        -----//
OPENSOAP_VAR
std::string
makeResponseIsEmptyMessage();

//----- attach message ID into SOAP message -----//
//----- return message ID -----//
//----- use in MsgDrv -----//
OPENSOAP_VAR
extern
std::string
attachMsgID(std::string& soapMsg, const std::string& fileIDOfMsgFromIf,
	    const std::string& operationName);

//----- attach message ID into SOAP message -----//
//----- return message ID -----//
//----- use in SrvDrv -----//
OPENSOAP_VAR
extern
std::string
attachOrUpdateMsgID(std::string& soapMsg, std::string msgIDContent);


//----- attach backward path to response message,                 -----//
//----- if this server is the final receiver of async fwd message -----//
//----- This is used by SrvDrv -----//
OPENSOAP_VAR
extern
bool
attachBackwardPathToResponse(std::string& msgFromService, 
			     std::string msgFromMsgDrv);

//----- attach ttl to response message,                 -----//
//----- if this server is the final receiver of async fwd message -----//
//----- This is used by SrvDrv -----//
OPENSOAP_VAR
extern
bool
attachTTLToResponse(std::string& msgFromService, 
		    std::string msgFromMsgDrv);

//attach received_path into SOAP message
OPENSOAP_VAR
extern
bool
attachReceivedPath(std::string& soapMsg, 
		   const std::string& fileIDOfMsgFromIf,
		   const std::string& url);

//attach or update backward_path SOAPmessage
OPENSOAP_VAR
extern
bool
updateBackwardPath(std::string& soapMsg, 
		   const std::string& backwardURL);

//----- Decrease value of <hopcount> in SOAP Message -----//
OPENSOAP_VAR
extern
bool
decreaseHopCount(std::string& soapMsg);

//attach message ID
//use service
OPENSOAP_VAR
extern
bool
attachMsgID(std::string& soapMsg, const std::string& msgIDContent);

//check this server is final receiver from request message
OPENSOAP_VAR
bool
isItAsyncFwdFinalReceiver(std::string soapMsg,
			  std::vector<std::string> soapServerEndPoints);

/*
  check backward_path
*/
OPENSOAP_VAR
bool
getBackwardPath(const std::string& soapMsg, std::string& backwardPath);


//check response message
// use MsgDrv, Forwarder
OPENSOAP_VAR
extern
bool
isItResponseMessage(std::string soapMsg);

//----- attach header for Service communication -----//
OPENSOAP_VAR
std::string
attachHeaderForService(std::string soapMsg);

//----- remove header for Service communication -----//
OPENSOAP_VAR
std::string
removeHeaderForService(std::string soapMsg);

//----- attach header SrvDrv communication -----//
OPENSOAP_VAR
std::string
attachHeaderForSrvDrv(std::string soapMsg);

//----- return hostname with FQDN -----//
OPENSOAP_VAR
std::string
getLocalhostName();

//----- check tag undelete ------//
OPENSOAP_VAR
bool
hasUndeleteTag(const std::string soapMsg);

#endif /* SOAP_MESSAGE_H */
