/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SSMLInfoFunc.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "SSMLInfoFunc.h"
#include "SSMLAttrHandler.h"

#include "ServerCommon.h"

using namespace OpenSOAP;
using namespace std;

bool
getOperationNameAndType(SSMLInfo& ssmlInfo)
{
  cerr << "===== SSMLInfoFunc::getOperationNameAndType() begin =====" << endl;
  bool operationExist = false;
  
  // operation name ̎擾
  std::string query;
  if (ssmlInfo.getNamespace().empty()) {
    query = "/SSML/service/operation=??"; 
  }
  else {
    query = "/SSML/service,nsuri='";
    query += ssmlInfo.getNamespace();
    query += "'/operation=??";
  }
  
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***:getOpNameAndType" 
	 << "query = [" << query << "]" << endl;
    return operationExist;
  }
  
  std::vector<std::string>::const_iterator pos;
  unsigned int operationNum = 0;
  for (pos = values.begin(); pos != values.end(); ++pos, ++operationNum) {
    cerr << "values = " << *pos << ", ";
    cerr << "ssmlInfo.operationName = " << ssmlInfo.getOperationName() << endl;
    if (*pos == ssmlInfo.getOperationName()) {
      operationExist = true;
      break;
    }
  }
  cerr << "SSMLInfoFunc::getOperationNameAndType: " 
       << "obtain operationType" << endl;
  // operation type ̎擾
  std::vector<std::string> values1;
  if (operationExist) {
    if (ssmlInfo.getNamespace().empty()) {
      query = "/SSML/service/operation,type=??"; 
    }
    else {
      query = "/SSML/service,nsuri='";
      query += ssmlInfo.getNamespace();
      query += "'/operation,type=??";
    }

    if (0 > ssmlAttrHndl.queryXml(query, values1)) {
      cerr << "*** ERROR ***" << "query = [" 
	   << query << "]" << endl;
      operationExist = false;
    }
    else {
      ssmlInfo.setOperationType(values1[operationNum]);
      cerr << "values1[operationNum] = " << values1[operationNum] << endl;
    }
  }
  
  cerr << "===== SSMLInfoFunc::getOperationNameAndType() end   =====" << endl;
  return operationExist;
}

void
getSyncTTL(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name=");
  query += ssmlInfo.getOperationType();
  query += "/synchronizedTTL=?";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setSyncTTL(0);
  }
  else {
    cerr << "values[0] = "  << values[0] << endl;
    ssmlInfo.setSyncTTL(atoi(values[0].c_str()));
  }
}

void
getAsyncTTL(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name="); 
  query += ssmlInfo.getOperationType();
  query += "/asynchronizedTTL=?";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setAsyncTTL(0);
  }
  else {
    cerr << "values[0] = "  << values[0] << endl;
    ssmlInfo.setAsyncTTL(atoi(values[0].c_str()));
  }
}

void
getHostname(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name=");
  query += ssmlInfo.getOperationType();
  query += "/Socket,hostname=?";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setHostname("");
  }
  else {
    cerr << "values[0] = "  << values[0] << endl;
    ssmlInfo.setHostname(values[0]);
  }
}

void
getPort(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name=");
  query += ssmlInfo.getOperationType();
  query += "/Socket,port=?";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setPort(0);
  }
  else {
    cerr << "values[0] = "  << values[0] << endl;
    ssmlInfo.setPort(atoi(values[0].c_str()));
  }
}

//added 2002.03.11
void
getExecProg(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name="); 
  query += ssmlInfo.getOperationType();
  query += "/StdIO/exec,prog=?";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setExecProg("");
  }
  else {
    cerr << "values[0] = "  << values[0] << endl;
    ssmlInfo.setExecProg(values[0]);
  }
}

void
getConnectionMethod(SSMLInfo& ssmlInfo)
{
  std::string query("/SSML/service/connection,name="); 
  query += ssmlInfo.getOperationType();
  query += "/??";
  SSMLAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << "query = [" 
	 << query << "]" << endl;
    ssmlInfo.setConnectionMethod("");
  }

  std::vector<std::string>::const_iterator pos;
  unsigned int operationNum = 0;
  for (pos = values.begin(); pos != values.end(); ++pos, ++operationNum) {
    cerr << "values = " << *pos << ", ";
	if (*pos == SRV_CONNCT_SOCKET ||
		*pos == SRV_CONNCT_STDIO  ||
		*pos == SRV_CONNCT_FIFO   ||
		*pos == SRV_CONNCT_IPC    ||
		*pos == SRV_CONNCT_MODULE ||
		*pos == SRV_CONNCT_COM) {

		ssmlInfo.setConnectionMethod(*pos);
		break;
	}
  }
}

extern
bool
getSSMLInfo(SSMLInfo& ssmlInfo)
{
  /* parse ssml begin */

  // Iy[Vƃ^Cv̎擾
  bool operationExist = getOperationNameAndType(ssmlInfo);
  
  // TTL ̎擾
  std::vector<std::string> timeValues;
  if (operationExist) {
    getSyncTTL(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getSyncTTL() = " << ssmlInfo.getSyncTTL() << endl;
    
    getAsyncTTL(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getAsyncTTL() = " << ssmlInfo.getAsyncTTL() << endl;

    getHostname(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getHostname() = " << ssmlInfo.getHostname() << endl;
    
    getPort(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getPort() = " << ssmlInfo.getPort() << endl;

	//added 2002.03.11
	getExecProg(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getExecProg() = " << ssmlInfo.getExecProg() << endl;

	getConnectionMethod(ssmlInfo);
    cerr << "SSMLInfoFunc::getSSMLInfo:"
	 << "ssmlInfo.getConnectionMethod() = " 
		<< ssmlInfo.getConnectionMethod() << endl;

  }
  /* parse ssml end */
  
  return operationExist;

}

