/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: SignatureFunc.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <string>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "fio.h"
#include "ServerCommon.h"
#include "SrvConfAttrHandler.h"
#include "OpenSOAP/OpenSOAP.h"
#include "OpenSOAP/Envelope.h"
#include "OpenSOAP/Security.h"

using namespace OpenSOAP;
using namespace std;

// check "conf/server/signature.conf" to sign or not
bool
needSignature()
{
  SrvConfAttrHandler ssmlAttrHndl;
  std::vector<std::string> values;
  std::string query;
  
  query = "/add_signature=?";
  if (0 > ssmlAttrHndl.queryXml(query, values)) {
    cerr << "*** ERROR ***" << endl;
    return false;
  }
  
  cerr << "### RESULT ###" << endl;
  cerr << "SignatureFunc::needSignature:"
       << "values[0] = " << values[0] << endl;
  if(values[0] == "true") {
    return true;
  }
  
  return false;
}

bool
addSignatureToFile(std::string fileID)
{
  
  // add signature
  std::string filename = MESSAGE_FILE_PATH + fileID;
  std::string szPrivKName = SECURITY_DATA_PATH + "privKey.pem";
  int nRet;
  OpenSOAPEnvelopePtr env = NULL;
  
  /* OpenSOAP API  */
  nRet = OpenSOAPInitialize(NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  /* Envelopeե */
  nRet = LoadEnvelope(filename.c_str(), &env);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  /* ̾ղ */
  nRet = OpenSOAPSecAddSignWithFile(env, OPENSOAP_HA_SHA,
				    szPrivKName.c_str(), NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  
  /* ̾Τ¸ */
  nRet = SaveEnvelope(env, filename.c_str());
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
FuncEnd:
  OpenSOAPUltimate();
  
  return true;
}

bool
addSignatureToString(std::string& soapMsg)
{
  std::string szPrivKName = SECURITY_DATA_PATH + "privKey.pem";
  int nRet;
  OpenSOAPEnvelopePtr env = NULL;
  
  /* OpenSOAP API  */
  nRet = OpenSOAPInitialize(NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    return false;
  }
  
  /* Envelopeʸ */
  nRet = LoadEnvelope(soapMsg, &env);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "LoadEnvelpe fail." << endl;
    return false;
  }
  
  /* ̾ղ */
  nRet = OpenSOAPSecAddSignWithFile(env, OPENSOAP_HA_SHA,
				    szPrivKName.c_str(), NULL);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "OpenSOAPSecAddSignWithFile fail." << endl;
    return false;
  }
  
  /* ̾Τ¸ */
  nRet = SaveEnvelope(env, soapMsg);
  if (!OPENSOAP_SUCCEEDED(nRet)) {
    cerr << "addSignatureToString(): "
	 << "SaveEnvelope fail." << endl;
    return false;
  }
  
FuncEnd:
  OpenSOAPUltimate();
  
  return true;
}
